@@ -1,6 +1,6 @@
 import sys, os, logging
 import gtk, gobject
-from sugar.activity.Activity import Activity
+from sugar.activity import activity
 
 #need to set things up before we import penguintv
 
@@ -9,11 +9,8 @@ try:
 except:
 	logging.warning("Trying to load bundled pycurl libraries")
 	
-	#import ourselves so we can get __file__
-	import PenguinTVActivity
-	
-	#where the hell are we?
-	activity_root = os.path.split(PenguinTVActivity.__file__)[0]
+	#where are we?
+	activity_root = activity.get_bundle_path()
 	
 	#chdir here so that relative RPATHs line up ('./lib')
 	os.chdir(activity_root) 
@@ -27,11 +24,11 @@ os.environ['SUGAR_PENGUINTV'] = '1' #set up variable so that utils knows we are
 
 from penguintv import penguintv
 
-class PenguinTVActivity(Activity):
-	def __init__(self):
-		Activity.__init__(self)
+class PenguinTVActivity(activity.Activity):
+	def __init__(self, handle):
+		activity.Activity.__init__(self, handle)
 		app = penguintv.PenguinTVApp(self)
-		self.set_title('PenguinTV')
+		self.set_title('News Reader')
 		self.connect('destroy',self.do_quit, app)
 	
 	def do_quit(self, event, app):
