@@ -12,10 +12,12 @@ import sys
 
 #loaded as needed
 #import feedparser
-#import HTMLParser 
+import HTMLParser 
 
 import utils
 import LoginDialog
+if utils.HAS_PYXML:
+	import itunes
 
 _=gettext.gettext
 
@@ -38,9 +40,9 @@ class AddFeedDialog:
 		self._feed_url_widget.set_text("")
 		self.set_location()
 		self._edit_tags_widget.set_text("")
-		if utils.RUNNING_SUGAR:
-			self._tag_hbox.hide()
-			self._label.set_text(_('Please enter the URL of the feed you would like to add'))
+		#if utils.RUNNING_SUGAR:
+		#	self._tag_hbox.hide()
+		#	self._label.set_text(_('Please enter the URL of the feed you would like to add'))
 	
 	#ripped from straw
 	def set_location(self):
@@ -63,14 +65,18 @@ class AddFeedDialog:
 		
 	def finish(self):
 		tags=[]
-		for tag in self._edit_tags_widget.get_text().split(','):
-			tags.append(tag.strip())
+		if len(self._edit_tags_widget.get_text()) > 0:
+			for tag in self._edit_tags_widget.get_text().split(','):
+				tags.append(tag.strip())
 		url = self._feed_url_widget.get_text()
 		self._window.set_sensitive(False)
 		while gtk.events_pending(): #make sure the sensitivity change goes through
 			gtk.main_iteration()
 		try:
 			url,title = self._correct_url(url)
+			if url is None:
+				self._window.set_sensitive(True)
+				return
 			feed_id = self._app.add_feed(url,title)
 		except AuthorizationFailed:
 			dialog = gtk.Dialog(title=_("Authorization Required"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
@@ -85,7 +91,7 @@ class AddFeedDialog:
 		except AuthorizationCancelled:
 			self._window.set_sensitive(True)
 			return
-		except BadFeedURL:
+		except BadFeedURL, e:
 			dialog = gtk.Dialog(title=_("No Feed in Page"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
 			label = gtk.Label(_("PenguinTV couldn't find a feed in the web page you provided.\nYou will need to find the RSS feed link in the web page yourself.  Sorry."))
 			dialog.vbox.pack_start(label, True, True, 0)
@@ -104,6 +110,9 @@ class AddFeedDialog:
 			return #don't hide, give them a chance to try again.
 		if len(tags) > 0:
 			self._app.apply_tags_to_feed(feed_id, None, tags)
+			#HACK: total hack to select the first tag they entered
+			#(tag order not preserved in DB, so we can't use the standard API
+			#self._app.main_window.select_feed(feed_id)
 			self._app.main_window.set_active_filter(self._app.main_window.get_filter_index(tags[0]))
 		self.hide()
 				
@@ -157,6 +166,13 @@ class AddFeedDialog:
 				self.tries+=1
 				return (d.username, d.password)
 				
+		if utils.HAS_PYXML:
+			if itunes.is_itunes_url(url):
+				try:
+					url = itunes.get_rss_from_itunes(url)
+				except:
+					raise BadFeedURL,"Error trying to get itunes podcast"
+				
 		urllib._urlopener = my_url_opener(gtk.glade.XML(self._app.glade_prefix+'/penguintv.glade', "dialog_login",'penguintv'))
 		url_stream = None
 		try:
@@ -199,14 +215,14 @@ class AddFeedDialog:
 		if mimetype in handled_mimetypes:
 			pass
 		elif mimetype in ['text/html', 'application/xhtml+xml']:
-			p = utils.AltParser()
+			p = AltParser()
 			try:
 				for line in url_stream.readlines():
 					p.feed(line)
 					if p.head_end: #if we've gotten an error, we need the whole page
 						break #otherwise the header is enough
 					
-				available_versions = p.alt_tags.keys()
+				available_versions = p.alt_tags
 				if len(available_versions)==0: #this might actually be a feed
 					data = feedparser.parse(url)
 					if len(data['channel']) == 0 or len(data['items']) == 0: #nope
@@ -216,10 +232,9 @@ class AddFeedDialog:
 						pass #we're good
 				else:
 					newurl=""
-					largest=0
-					for m in handled_mimetypes:
-						if m in available_versions:
-							pos_url = p.alt_tags[m]
+					url_choices = []
+					for mimetype, pos_url, t in available_versions:
+						if mimetype in handled_mimetypes:
 							#first clean it up
 							if pos_url[:4]!="http": #maybe the url is not fully qualified (fix for metaphilm.com)
 								if pos_url[0:2] == '//': #fix for gnomefiles.org
@@ -231,13 +246,12 @@ class AddFeedDialog:
 									pos_url=os.path.split(url)[0]+'/'+pos_url
 									
 							#now test sizes
-							try:
-							    size = len(urllib.urlopen(pos_url).read())
-							except:
-							    continue
-							if size > largest:
-								newurl = pos_url
-								
+							url_choices.append((pos_url, t))
+							
+					if len(url_choices) > 1:
+						newurl, title = self._choose_url(url_choices)
+					elif len(url_choices) == 1:
+						newurl, title = url_choices[0]
 					if newurl == "":
 						print "warning: unhandled alt mimetypes:"+str(p.alt_tags)
 						raise BadFeedURL
@@ -257,6 +271,62 @@ class AddFeedDialog:
 			print "warning: unhandled page mimetypes: "+str(mimetype)+"<--"
 			raise BadFeedURL
 		return (url,title)
+		
+	def _choose_url(self, url_list):
+		dialog = gtk.Dialog(title=_("Choose Feed"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+
+		label = gtk.Label(_("Please choose one of the feeds in this page"))
+		dialog.vbox.pack_start(label, True, True, 0)
+		
+		list_widget = gtk.TreeView()
+		model = gtk.ListStore(str, str)
+		r = gtk.CellRendererText()
+		c = gtk.TreeViewColumn('Feeds')
+		c.pack_start(r)
+		c.set_attributes(r, markup=1)
+		list_widget.append_column(c)
+		list_widget.set_model(model)
+		dialog.vbox.pack_start(list_widget)
+		
+		for url, title in url_list:
+			model.append([url, title])
+		
+		dialog.show_all()
+		response = dialog.run()
+		dialog.hide()
+		del dialog
+		if response == gtk.RESPONSE_ACCEPT:	
+			selection = list_widget.get_selection()
+			s_iter = selection.get_selected()[1]
+			if s_iter is None:
+				return None
+			return list(model[s_iter])
+		return None
+
+class AltParser(HTMLParser.HTMLParser):
+	def __init__(self):
+		HTMLParser.HTMLParser.__init__(self)
+		self.alt_tags=[]
+		self.head_end=False
+		
+	def handle_starttag(self, tag, attrs):
+		"""Signal when we get to a tag."""
+		if tag=='link':
+			attr_dic = {}
+			for attr in attrs:
+				attr_dic[attr[0]] = attr[1]
+			try:
+				if attr_dic['rel'] == 'alternate':
+					if attr_dic['type'] in ['application/atom+xml','application/rss+xml','text/xml']:
+						attr_dic.setdefault('title',attr_dic['href'])
+						self.alt_tags.append((attr_dic['type'], attr_dic['href'], attr_dic['title']))
+			except:
+				pass
+
+	def handle_endtag(self, tag):
+		if tag == 'head':
+			self.head_end=True
+
 
 class AuthorizationFailed(Exception):
 	def __init__(self):
