@@ -11,19 +11,15 @@ import formatter
 import threading
 import re
 
-#import traceback
-
 try:
 	#not good enough to load it below.  need to load it module-wide
 	#or else random images don't load.  gtkmozembed is VERY picky!
 	import gtkmozembed
-	MOZ_OK = True
 except:
-	MOZ_OK = False
-
+	pass
+	
 GTKHTML=0
 MOZILLA=1
-DEMOCRACY_MOZ=2
 
 #states
 S_DEFAULT = 0
@@ -68,10 +64,6 @@ class EntryView(gobject.GObject):
 			self._current_scroll_v = scrolled_window.get_vadjustment().get_value()
 			self._current_scroll_h = scrolled_window.get_hadjustment().get_value()
 			self._scrolled_window = scrolled_window
-		elif self._renderer == MOZILLA:
-			import gtkmozembed
-		elif self._renderer == DEMOCRACY_MOZ:
-			from democracy_moz import MozillaBrowser
 				
 		#thanks to straw, again
 		style = html_dock.get_style().copy()
@@ -127,6 +119,8 @@ class EntryView(gobject.GObject):
 			f = open (os.path.join(self._app.glade_prefix,"mozilla.css"))
 			for l in f.readlines(): self._css += l
 			f.close()
+			if not utils.init_gtkmozembed():
+				print "Error initializing mozilla.  Penguintv may crash shortly"
 			gtkmozembed.set_profile_path(self._db.home, 'gecko')
 			self._moz = gtkmozembed.MozEmbed()
 			self._moz.connect("open-uri", self._moz_link_clicked)
@@ -142,23 +136,6 @@ class EntryView(gobject.GObject):
 				self._conf = gconf.client_get_default()
 				self._conf.notify_add('/desktop/gnome/interface/font_name',self._gconf_reset_moz_font)
 			self._reset_moz_font()
-		elif self._renderer==DEMOCRACY_MOZ:
-			f = open (os.path.join(self._app.glade_prefix,"mozilla.css"))
-			for l in f.readlines(): self._css += l
-			f.close()
-			self._mb = MozillaBrowser.MozillaBrowser()
-			self._moz = self._mb.getWidget()
-			self._moz.connect("link-message", self._moz_link_message)
-			self._moz.connect("realize", self._moz_realize, True)
-			self._moz.connect("unrealize", self._moz_realize, False)
-			self._mb.setURICallBack(self._dmoz_link_clicked)
-			self._moz.load_url("about:blank")
-			scrolled_window.add_with_viewport(self._moz)
-			if utils.HAS_GCONF:
-				import gconf
-				self._conf = gconf.client_get_default()
-				self._conf.notify_add('/desktop/gnome/interface/font_name',self._gconf_reset_moz_font)
-			self._reset_moz_font()
 			
 		html_dock.show_all()
 		#self.display_custom_entry("<html></html>")
@@ -200,6 +177,7 @@ class EntryView(gobject.GObject):
 	
 	def _moz_realize(self, widget, realized):
 		self._moz_realized = realized
+		self.display_item()
 		 
 	def _dmoz_link_clicked(self, link):
 		link = link.strip()
@@ -268,7 +246,7 @@ class EntryView(gobject.GObject):
             body { background-color: %s; }</style><body>%s</body></html>""" % (self._background_color,message))
 			self._document.close_stream()
 			self._document_lock.release()
-		elif self._renderer==MOZILLA or self._renderer == DEMOCRACY_MOZ:
+		elif self._renderer==MOZILLA:
 			if self._moz_realized:
 				self._moz.open_stream("http://ywwg.com","text/html")
 				while len(message)>60000:
@@ -291,7 +269,7 @@ class EntryView(gobject.GObject):
 				self._document.write_stream(message)
 				self._document.close_stream()
 				self._document_lock.release()
-			elif self._renderer==MOZILLA or self._renderer == DEMOCRACY_MOZ:
+			elif self._renderer==MOZILLA:
 				if self._moz_realized:
 					self._moz.open_stream("http://ywwg.com","text/html")
 					self._moz.append_data(message, long(len(message)))
@@ -352,7 +330,7 @@ class EntryView(gobject.GObject):
 				self._current_scroll_v = va.get_value()
 				self._current_scroll_h = ha.get_value()	
 	
-		if self._renderer == MOZILLA or self._renderer == DEMOCRACY_MOZ:
+		if self._renderer == MOZILLA:
 			if item is not None:
 				#no comments in css { } please!
 				#FIXME windows: os.path.join... wrong direction slashes?  does moz care?
@@ -440,7 +418,7 @@ class EntryView(gobject.GObject):
 			else:
 				va.set_value(va.lower)
 				ha.set_value(ha.lower)
-		elif self._renderer == MOZILLA or self._renderer == DEMOCRACY_MOZ:	
+		elif self._renderer == MOZILLA:	
 			if self._moz_realized:
 				self._moz.open_stream("http://ywwg.com","text/html") #that's a base uri for local links.  should be current dir
 				while len(html)>60000:
@@ -495,7 +473,7 @@ class EntryView(gobject.GObject):
             body { background-color: %s; }</style><body></body></html>""" % (self._insensitive_color,))
 			self._document.close_stream()
 			self._document_lock.release()
-		elif self._renderer==MOZILLA or self._renderer == DEMOCRACY_MOZ:
+		elif self._renderer==MOZILLA:
 			#FIXME: this doesn't work, we quit before it renders
 			message = """<html><head><style type="text/css">
             body { background-color: %s; }</style></head><body></body></html>""" % (self._insensitive_color,)
@@ -504,7 +482,7 @@ class EntryView(gobject.GObject):
 		self._custom_entry = True
 		return
 		
-def htmlify_item(item, mm=None, ajax=False, with_feed_titles=False, indicate_new=False):
+def htmlify_item(item, mm=None, ajax=False, with_feed_titles=False, indicate_new=False, basic_progress=False):
 	""" Take an item as returned from ptvDB and turn it into an HTML page.  Very messy at times,
 	    but there are lots of alternate designs depending on the status of media. """
 
@@ -535,22 +513,22 @@ def htmlify_item(item, mm=None, ajax=False, with_feed_titles=False, indicate_new
 		ret.append('<div class="sdate">%s</div>' % time.strftime('%a %b %d, %Y %X',time.localtime(item['date'])))
 		#ret.append('</div>')
 	if item.has_key('media'):
-		
 		if mm is not None and not ajax:
 			for medium in item['media']:
-				ret += htmlify_media(medium, mm)
+				ret += htmlify_media(medium, mm, basic_progress)
 		else:
 			ret += '<span id="' + str(item['entry_id']) + '"></span>'
 	ret.append('<div class="content">')
 	if item.has_key('description'):
+		#ret.append('%s' % item['description'].replace('\n\n', '<br/><br/>'))
 		ret.append('%s' % item['description'])
 	ret.append('</div>')
 	if item.has_key('link'):
-		ret.append('<a href="'+item['link']+'">'+_("Full Entry...")+'</a>' )
+		ret.append('<a href="' + item['link'] + '">' + _("Full Entry...") + '</a>')
 	ret.append('</p></div>')
 	return "".join(ret)
 	
-def htmlify_media(medium, mm):
+def htmlify_media(medium, mm, basic_progress=False):
 	ret = []
 	ret.append('<div class="media">')
 	if medium['download_status']==ptvDB.D_NOT_DOWNLOADED:    
@@ -558,7 +536,9 @@ def htmlify_media(medium, mm):
 						 utils.html_command('downloadqueue:',medium['media_id'])+
 				         ' (%s)</p>' % (utils.format_size(medium['size'],)))
 	elif medium['download_status'] == ptvDB.D_DOWNLOADING: 
-		if medium.has_key('progress_message'): #downloading and we have a custom message
+		if basic_progress:
+			ret.append('<p><i>'+_('Downloading %s...') % utils.format_size(medium['size'])+'</i> '+utils.html_command('pause:',medium['media_id'])+' '+utils.html_command('stop:',medium['media_id'])+'</p>')
+		elif medium.has_key('progress_message'): #downloading and we have a custom message
 			ret.append('<p><i>'+medium['progress_message']+'</i> '+
 			                    utils.html_command('pause:',medium['media_id'])+' '+
 			                    utils.html_command('stop:',medium['media_id'])+'</p>')
@@ -581,8 +561,7 @@ def htmlify_media(medium, mm):
 			            utils.html_command('pause:',medium['media_id'])+' '+
 			            utils.html_command('stop:',medium['media_id'])+'</p>')
 		else:       # we have nothing to go on
-			ret.append('<p><i>'+_('Downloading %s...') % utils.format_size(medium['size'])+'</i> '+utils.html_command('pause:',medium['media_id'])+' '+
-													  utils.html_command('stop:',medium['media_id'])+'</p>')
+			ret.append('<p><i>'+_('Downloading %s...') % utils.format_size(medium['size'])+'</i> '+utils.html_command('pause:',medium['media_id'])+' '+utils.html_command('stop:',medium['media_id'])+'</p>')
 	elif medium['download_status'] == ptvDB.D_DOWNLOADED:
 		if mm.has_downloader(medium['media_id']):	
 			downloader = mm.get_downloader(medium['media_id'])
