@@ -7,7 +7,7 @@ import utils
 import IconManager
 import MainWindow
 
-import traceback, sys, os, re
+import sys, os, re
 import glob
 
 import random
@@ -87,6 +87,7 @@ class FeedList(gobject.GObject):
 		self.filter_setting=ALL
 		self.filter_name = _("All Feeds")
 		self._selecting_misfiltered=False
+		self._filter_unread = False
 		self._cancel_load = [False,False] #loading feeds, loading details
 		self._loading_details = False
 		self._state = S_DEFAULT
@@ -143,10 +144,14 @@ class FeedList(gobject.GObject):
 		self._handlers = []
 		h_id = self._app.connect('feed-polled', self.__feed_polled_cb)
 		self._handlers.append((self._app.disconnect, h_id))
+		h_id = self._app.connect('feed-added', self.__feed_added_cb)
+		self._handlers.append((self._app.disconnect, h_id))
 		h_id = self._app.connect('feed-removed', self.__feed_removed_cb)
 		self._handlers.append((self._app.disconnect, h_id))
 		h_id = self._app.connect('entry-updated', self.__entry_updated_cb)
 		self._handlers.append((self._app.disconnect, h_id))
+		h_id = self._app.connect('tags-changed', self.__tags_changed_cb)
+		self._handlers.append((self._app.disconnect, h_id))
 		
 		#init style
 		if self._fancy:
@@ -160,13 +165,23 @@ class FeedList(gobject.GObject):
 		for disconnector, h_id in self._handlers:
 			disconnector(h_id)
 			
-	def __feed_polled_cb(self, app, feed_id):
-		self.update_feed_list(feed_id,['readinfo','icon','title','image'])
+	def __feed_polled_cb(self, app, feed_id, update_data):
+		if update_data.has_key('no_changes'):
+			if update_data['no_changes']:
+				self.update_feed_list(feed_id, ['icon'], update_data)
+				return
+		self.update_feed_list(feed_id, ['readinfo','icon','title','image'], update_data)
+		
+	def __feed_added_cb(self, app, feed_id, success):
+		self.update_feed_list(feed_id, ['title'])
 			
 	def __feed_removed_cb(self, app, feed_id):
 		self.remove_feed(feed_id)
 		self.resize_columns()
 		
+	def __tags_changed_cb(self, app, a):
+		self.filter_all(False)
+	
 	def __entry_updated_cb(self, app, entry_id, feed_id):
 		self.update_feed_list(feed_id,['readinfo','icon'])	
 			
@@ -335,7 +350,7 @@ class FeedList(gobject.GObject):
 		if feed_id is None:
 			if self._last_feed is None:
 				return
-			feed_id = self._last_feed
+			feed_id = self._feedlist[self._last_feed][FEEDID]
 			
 		if update_what is None:
 			update_what = ['readinfo','icon','title']
@@ -355,11 +370,7 @@ class FeedList(gobject.GObject):
 		
 		if 'title' in update_what or 'icon' in update_what:
 			if not update_data.has_key('flag_list'):
-				entrylist = self._db.get_entrylist(feed_id)
-				if entrylist:
-					update_data['flag_list'] = self._db.get_entry_flags(feed_id)
-				else:
-					update_data['flag_list'] = []
+				update_data['flag_list'] = self._db.get_entry_flags(feed_id)
 						
 			updated=0
 			unviewed=0
@@ -392,6 +403,10 @@ class FeedList(gobject.GObject):
 			if self._fancy:
 				readinfo_string += "\n"
 			feed[READINFO] = self._get_markedup_title(readinfo_string,flag)
+			
+			if self._filter_unread: 	 
+				if unviewed==0 and self.filter_test_feed(feed_id): #no sense testing the filter if we won't see it 	 
+					need_filter = True
 				
 		if 'title' in update_what:
 			selected = self.get_selected()
@@ -399,25 +414,29 @@ class FeedList(gobject.GObject):
 				update_data['title'] = self._db.get_feed_title(feed_id)
 			old_title_len = len(feed[TITLE])
 			new_title_len = len(update_data['title'])
-			feed[TITLE] = update_data['title']
+			
+			# don't update feed[TITLE] yet, we need these data first
 			if self._fancy:
 				try: feed[FIRSTENTRYTITLE] = self._db.get_entrylist(feed_id)[0][1]
 				except: feed[FIRSTENTRYTITLE] = ""
-				feed[MARKUPTITLE] = self._get_fancy_markedup_title(feed[TITLE],feed[FIRSTENTRYTITLE],feed[UNREAD], feed[TOTAL], flag, feed_id == selected)
+				feed[MARKUPTITLE] = self._get_fancy_markedup_title(update_data['title'],feed[FIRSTENTRYTITLE],feed[UNREAD], feed[TOTAL], flag, feed_id == selected)
 			else:
-				feed[MARKUPTITLE] = self._get_markedup_title(feed[TITLE], flag)
-			try:
-				old_iter = self._feedlist.get_iter((self.find_index_of_item(feed_id),))
-				new_iter = self._feedlist.get_iter(([f[0] for f in self._db.get_feedlist()].index(feed_id),))
-				self._feedlist.move_after(old_iter,new_iter)
-				if selected == feed_id:
-					self._widget.scroll_to_cell((self.find_index_of_item(feed_id),))
-			except:
-				print "Error finding feed for update"
-			need_filter = True
-			#columns_autosize produces a flicker, so only do it if we need to
-			if abs(new_title_len - old_title_len) > 5:
-				self._widget.columns_autosize()
+				feed[MARKUPTITLE] = self._get_markedup_title(update_data['title'], flag)
+				
+			if feed[TITLE] != update_data['title']:
+				feed[TITLE] = update_data['title']
+				try:
+					old_iter = self._feedlist.get_iter((self.find_index_of_item(feed_id),))
+					new_iter = self._feedlist.get_iter(([f[0] for f in self._db.get_feedlist()].index(feed_id),))
+					self._feedlist.move_after(old_iter,new_iter)
+					if selected == feed_id:
+						self._widget.scroll_to_cell((self.find_index_of_item(feed_id),))
+				except:
+					print "Error finding feed for update"
+				need_filter = True
+				#columns_autosize produces a flicker, so only do it if we need to
+				if abs(new_title_len - old_title_len) > 5:
+					self._widget.columns_autosize()
 			
 		if 'icon' in update_what:
 			if not update_data.has_key('pollfail'):
@@ -440,22 +459,21 @@ class FeedList(gobject.GObject):
 		"""alters the number of unread entries by num_to_mark.  if negative,
 		marks some as unread"""
 		
-		s = self._widget.get_selection().get_selected()
-		assert s is not None
-		model, iter = s
-		if iter is None:
-			return None
-		path = model.get_path(iter)
-		index = path[0]
-		selected = model[index][FEEDID]
-
 		#there's some trickiness here.  The model for the selection is
 		#self._feed_filter, not self._feedlist, so we can't write to items
 		#in that model.  We have to go back and find where this feed is in the
 		#original model.
 
 		if feed_id is None:
-			feed = self._feedlist[self.find_index_of_item(selected)]
+			s = self._widget.get_selection().get_selected()
+			if s is None:
+				return
+			model, iter = s
+			if iter is None:
+				return
+
+			unfiltered_iter = model.convert_iter_to_child_iter(iter)
+			feed = self._feedlist[unfiltered_iter]
 		else:
 			feed = self._feedlist[self.find_index_of_item(feed_id)]
 
@@ -463,7 +481,7 @@ class FeedList(gobject.GObject):
 
 		#sanity check
 		if feed[UNREAD] - num_to_mark < 0 or feed[UNREAD] - num_to_mark > feed[TOTAL]:
-			print "WARNING: trying to mark more or less than we have:", feed[UNREAD], num_to_mark
+			print "WARNING: trying to mark more or less than we have:", feed[TITLE], feed[UNREAD], num_to_mark
 			self.update_feed_list(feed[FEEDID], ['readinfo'])
 			return
 			
@@ -484,11 +502,14 @@ class FeedList(gobject.GObject):
 															   feed[UNREAD], 
 															   feed[TOTAL], 
 															   feed[FLAG], 
-															   feed[FEEDID] == selected)
+															   True)
 		else:
 			feed[MARKUPTITLE] = self._get_markedup_title(feed[TITLE], feed[FLAG])
 			
 		feed[READINFO] = self._get_markedup_title(readinfo_string,feed[FLAG])
+		
+		if self._filter_unread: 	 
+			self._filter_one(feed)
 				
 	def show_search_results(self, results=[]):
 		
@@ -587,8 +608,8 @@ class FeedList(gobject.GObject):
 		self._unset_state(data)
 		self._state = newstate
 			
-	def filter_all(self,keep_misfiltered=True):
-		if self.filter_setting == SEARCH:
+	def filter_all(self, keep_misfiltered=True):
+		if utils.HAS_LUCENE and self.filter_setting == SEARCH:
 			print "not filtering, we have search results"
 			return #not my job
 	
@@ -618,7 +639,10 @@ class FeedList(gobject.GObject):
 			if i == index and selected is not None:  #if it's the selected feed, we have to be careful
 				if keep_misfiltered: 
 					#some cases when we want to keep the current feed visible
-					if self.filter_setting == DOWNLOADED and flag & ptvDB.F_DOWNLOADED == 0 and flag & ptvDB.F_PAUSED == 0:
+					if self._filter_unread == True and flag & ptvDB.F_UNVIEWED==0: #if it still fails the unviewed test
+						passed_filter = True  #keep it
+						self._selecting_misfiltered=True
+					elif self.filter_setting == DOWNLOADED and flag & ptvDB.F_DOWNLOADED == 0 and flag & ptvDB.F_PAUSED == 0:
 						passed_filter = True
 						self._selecting_misfiltered=True
 					elif self.filter_setting == DOWNLOADED and flag & ptvDB.F_DOWNLOADING:
@@ -628,6 +652,9 @@ class FeedList(gobject.GObject):
 					self._widget.get_selection().unselect_all() #and clear out the entry list and entry view
 					#self._app.display_feed(-1)
 					self.emit('no-feed-selected')
+			else: #if it's not the selected feed
+				if self._filter_unread == True and flag & ptvDB.F_UNVIEWED==0: #and it fails unviewed
+					passed_filter = False #see ya
 			if feed[VISIBLE] != passed_filter:
 				feed[VISIBLE] = passed_filter #note, this seems to change the selection!
 		self._feed_filter.refilter()
@@ -635,7 +662,7 @@ class FeedList(gobject.GObject):
 		return False
 
 	def _filter_one(self,feed, keep_misfiltered=True):
-		if self.filter_setting == SEARCH:
+		if utils.HAS_LUCENE and self.filter_setting == SEARCH:
 			print "not filtering, we have search results"
 			return #not my job
 	
@@ -661,7 +688,10 @@ class FeedList(gobject.GObject):
 		if feed_index == s_index and selected is not None:  #if it's the selected feed, we have to be careful
 			if keep_misfiltered: 
 				#some cases when we want to keep the current feed visible
-				if self.filter_setting == DOWNLOADED and flag & ptvDB.F_DOWNLOADED == 0 and flag & ptvDB.F_PAUSED == 0:
+				if self._filter_unread == True and flag & ptvDB.F_UNVIEWED==0: #if it still fails the unviewed test
+					passed_filter = True  #keep it
+					self._selecting_misfiltered=True
+				elif self.filter_setting == DOWNLOADED and flag & ptvDB.F_DOWNLOADED == 0 and flag & ptvDB.F_PAUSED == 0:
 					passed_filter = True
 					self._selecting_misfiltered=True
 				elif self.filter_setting == DOWNLOADED and flag & ptvDB.F_DOWNLOADING:
@@ -671,6 +701,9 @@ class FeedList(gobject.GObject):
 				self._widget.get_selection().unselect_all() #and clear out the entry list and entry view
 				#self._app.display_feed(-1)
 				self.emit('no-feed-selected')
+		else: #if it's not the selected feed
+			if self._filter_unread == True and flag & ptvDB.F_UNVIEWED==0: #and it fails unviewed
+				passed_filter = False #see ya
 		if feed[VISIBLE] != passed_filter:
 			feed[VISIBLE] = passed_filter #note, this seems to change the selection!
 		self._feed_filter.refilter()
@@ -750,8 +783,7 @@ class FeedList(gobject.GObject):
 			
 		self.filter_all(False)
 		if self._fancy:
-			if not self._loading_details:
-				gobject.idle_add(self._load_visible_details().next)
+			gobject.idle_add(self._load_visible_details().next)
 		self._va.set_value(0)
 		self.resize_columns()
 		
@@ -779,6 +811,15 @@ class FeedList(gobject.GObject):
 		self.populate_feeds(self._app._done_populating)
 		self._widget.columns_autosize()
 		
+	def set_unread_toggle(self, active):
+		if self._state == S_SEARCH:
+			return 
+		self._filter_unread = active
+		self.filter_all(False)
+		if self._fancy:
+			gobject.idle_add(self._load_visible_details().next)
+		self._va.set_value(0)	
+		
 	def clear_list(self):
 		self._feedlist.clear()
 		
@@ -846,6 +887,7 @@ class FeedList(gobject.GObject):
 			height = MIN_SIZE
 			width = p.get_width() * height / p.get_height()
 		if height != p.get_height() or width != p.get_width():
+			del p
 			p = gtk.gdk.pixbuf_new_from_file_at_size(filename, width, height)
 		return p
 		
@@ -908,59 +950,61 @@ class FeedList(gobject.GObject):
 			return best_flag
 			
 	def _item_selection_changed(self, selection):
-		item = self.get_selected()
-		
-		if item is None:
+		s = selection.get_selected()
+		if s:
+			model, iter = s
+			if iter is None:
+				self.emit('no-feed-selected') 
+				return
+			unfiltered_iter = model.convert_iter_to_child_iter(iter)
+			feed = self._feedlist[unfiltered_iter]
+		else:
 			self.emit('no-feed-selected') 
-		
-		try:
-			feed = self._feedlist[self.find_index_of_item(item)]
-		except:
 			return
 		
 		if self._fancy and self._last_feed is not None:
 			try: 
-				old_item = self._feedlist[self.find_index_of_item(self._last_feed)]
+				old_item = self._feedlist[self._last_feed]
 				old_item[MARKUPTITLE] = self._get_fancy_markedup_title(old_item[TITLE],old_item[FIRSTENTRYTITLE],old_item[UNREAD], old_item[TOTAL], old_item[FLAG], False)
 			except:
 				pass
 			
-		self._last_feed=item
+		self._last_feed=unfiltered_iter
 		self._select_after_load=None
 		
 		if self._fancy:
 			feed[MARKUPTITLE] = self._get_fancy_markedup_title(feed[TITLE],feed[FIRSTENTRYTITLE],feed[UNREAD], feed[TOTAL], feed[FLAG], True)
-	
+
+		try:
+			if self._feedlist[self.find_index_of_item(feed[FEEDID])][POLLFAIL]:
+				self._app.display_custom_entry("<b>"+_("There was an error trying to poll this feed.")+"</b>")
+			else:
+				self._app.undisplay_custom_entry()
+		except:
+			self._app.undisplay_custom_entry()
+
 		#if self._showing_search:
 		if self._state == S_SEARCH:
-			if item == self._last_selected:
+			if feed[FEEDID] == self._last_selected:
 				return
-			self._last_selected = item
+			self._last_selected = feed[FEEDID]
 			if not self._app.entrylist_selecting_right_now():
-				highlight_count = self._app.highlight_entry_results(item)
+				highlight_count = self._app.highlight_entry_results(feed[FEEDID])
 				if highlight_count == 0:
-					#self._app.display_feed(item)
-					self.emit('feed-selected', item)
+					#self._app.display_feed(feed[FEEDID])
+					self.emit('feed-selected', feed[FEEDID])
 			return
-		if item == self._last_selected:
-			#self._app.display_feed(item)
-			self.emit('feed-selected', item)
+		if feed[FEEDID] == self._last_selected:
+			#self._app.display_feed(feed[FEEDID])
+			self.emit('feed-selected', feed[FEEDID])
 		else:
-			self._last_selected = item
+			self._last_selected = feed[FEEDID]
 			#print "wtf is this negative 2 shit (feedlistview)"
-			#self._app.display_feed(item, -2)
-			self.emit('feed-selected', item)
-			if self._selecting_misfiltered and item!=None:
+			#self._app.display_feed(feed[FEEDID], -2)
+			self.emit('feed-selected', feed[FEEDID])
+			if self._selecting_misfiltered and feed[FEEDID]!=None:
 				self._selecting_misfiltered = False
 				gobject.timeout_add(250, self.filter_all)
-		try:
-			if self._feedlist[self.find_index_of_item(item)][POLLFAIL]:
-				self._app.display_custom_entry("<b>"+_("There was an error trying to poll this feed.")+"</b>")
-				return
-		except:
-			pass
-			
-		self._app.undisplay_custom_entry()
 			
 	def get_selected(self, selection=None):
 		if selection==None:
