@@ -30,6 +30,16 @@ else:
 	RUNNING_SUGAR = False
 
 class GStreamerPlayer(gobject.GObject):
+
+	__gsignals__ = {
+		'playing': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([])),
+		'paused': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([]))
+	}
+
 	def __init__(self, layout_dock):
 		gobject.GObject.__init__(self)
 		self._layout_dock = layout_dock
@@ -41,13 +51,16 @@ class GStreamerPlayer(gobject.GObject):
 		self.__no_seek = False
 		self.__is_exposed = False
 		self._x_overlay = None
+		self._prepare_save = False
 		
 		self._error_dialog = GStreamerErrorDialog()
 		
-		gobject.signal_new('item-queued', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [str,str])
-		gobject.signal_new('item-not-supported', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [str,str])
+		gobject.signal_new('item-queued', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [str, str, gobject.TYPE_PYOBJECT])
+		gobject.signal_new('item-not-supported', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [str, str, gobject.TYPE_PYOBJECT])
 		gobject.signal_new('items-removed', GStreamerPlayer, gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
 		
+		gobject.timeout_add(300000, self._periodic_save_cb)
+		
 	###public functions###
 		
 	def Show(self):
@@ -57,14 +70,15 @@ class GStreamerPlayer(gobject.GObject):
 		self._hpaned = gtk.HPaned()
 		self._player_vbox = gtk.VBox()
 		self._drawing_area = gtk.DrawingArea()
-		color = gtk.gdk.Color(0,0,0)
+		
+		color = gtk.gdk.Color(0, 0, 0)
 		self._drawing_area.modify_bg(gtk.STATE_NORMAL, color)
 		self._drawing_area.connect('expose-event', self._on_drawing_area_exposed)
 		self._player_vbox.pack_start(self._drawing_area)
 		vbox.pack_start(self._player_vbox, True)
 		
 		self._seek_scale = gtk.HScale()
-		self._seek_scale.set_range(0,1)
+		self._seek_scale.set_range(0, 1)
 		self._seek_scale.set_draw_value(False)
 		self._seek_scale.connect('value-changed', self._on_seek_value_changed)
 		vbox.pack_start(self._seek_scale, False)
@@ -76,7 +90,7 @@ class GStreamerPlayer(gobject.GObject):
 		s_w.set_shadow_type(gtk.SHADOW_IN)
 		s_w.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 		self._queue_listview = gtk.TreeView()
-		model = gtk.ListStore(str, str, str) #uri, title to display, current track indicator
+		model = gtk.ListStore(str, str, str, gobject.TYPE_PYOBJECT) #uri, title to display, current track indicator, user data
 		self._queue_listview.set_model(model)
 		
 		column = gtk.TreeViewColumn(_(""))
@@ -114,7 +128,6 @@ class GStreamerPlayer(gobject.GObject):
 		main_vbox.add(self._hpaned)
 		
 		
-		
 		self._controls_hbox = gtk.HBox()
 		self._controls_hbox.set_spacing(6)
 		button_box = gtk.HButtonBox()
@@ -190,6 +203,9 @@ class GStreamerPlayer(gobject.GObject):
 	def get_widget(self):
 		return self._layout_dock
 		
+	def is_exposed(self):
+		return self.__is_exposed
+		
 	def detach(self):
 		"""video window can detach.  queue stays embedded"""
 		pass
@@ -226,10 +242,10 @@ class GStreamerPlayer(gobject.GObject):
 		self._media_duration = pickle.load(playlist)
 		l = pickle.load(playlist)
 		model = self._queue_listview.get_model()
-		for uri, name in l:
-			model.append([uri, name, ""])
+		for uri, name, userdata in l:
+			model.append([uri, name, "", userdata])
 			filename = gst.uri_get_location(uri)
-			self.emit('item-queued', filename, name)
+			self.emit('item-queued', filename, name, userdata)
 		if self.__is_exposed:
 			self._seek_to_saved_position()
 		playlist.close()
@@ -251,12 +267,12 @@ class GStreamerPlayer(gobject.GObject):
 		pickle.dump(self._media_position, playlist)
 		pickle.dump(self._media_duration, playlist)
 		l = []
-		for uri, name, current in self._queue_listview.get_model():
-			l.append([uri,name])
+		for uri, name, current, userdata in self._queue_listview.get_model():
+			l.append([uri, name, userdata])
 		pickle.dump(l, playlist)
 		playlist.close()
 		
-	def queue_file(self, filename, name=None):
+	def queue_file(self, filename, name=None, userdata=None):
 		try:
 			os.stat(filename)
 		except:
@@ -268,12 +284,15 @@ class GStreamerPlayer(gobject.GObject):
 		
 		#thanks gstfile.py
 		self.current = Discoverer(filename)
-		self.current.connect('discovered', self._on_type_discovered, filename, name)
+		self.current.connect('discovered', self._on_type_discovered, filename, name, userdata)
 		self.current.discover()	
 			
 	def get_queue_count(self):
 		return len(self._queue_listview.get_model())
 		
+	def get_queue(self):
+		return list(self._queue_listview.get_model())
+		
 	def play_pause_toggle(self):
 		if self._pipeline.get_state()[1] == gst.STATE_PLAYING:
 			self.pause()
@@ -285,7 +304,7 @@ class GStreamerPlayer(gobject.GObject):
 		if len(model) == 0:
 			return
 		if self._current_index < 0: self._current_index = 0
-		uri, title, current = list(model[self._current_index])
+		uri, title, current, userdata = list(model[self._current_index])
 		if self._last_index != self._current_index:
 			self._last_index = self._current_index
 			selection = self._queue_listview.get_selection()
@@ -293,12 +312,13 @@ class GStreamerPlayer(gobject.GObject):
 			for row in model:
 				i+=1
 				if i == self._current_index:
-					row[2] = "&#8226;"
+					row[2] = "&#8226;" #bullet
 				else:
 					row[2] = ""
 			if not self._ready_new_uri(uri):
 				return
 			self._prepare_display()
+			self._prepare_save = True
 		self._pipeline.set_state(gst.STATE_PLAYING)
 		image = gtk.Image()
 		image.set_from_stock("gtk-media-pause",gtk.ICON_SIZE_BUTTON)
@@ -307,6 +327,7 @@ class GStreamerPlayer(gobject.GObject):
 		if not notick:
 			gobject.timeout_add(500, self._tick)
 		#self._pipeline.get_property('stream-info')
+		self.emit('playing')
 		
 	def pause(self):
 		try: self._media_position = self._pipeline.query_position(gst.FORMAT_TIME)[0]
@@ -315,12 +336,13 @@ class GStreamerPlayer(gobject.GObject):
 		image = gtk.Image()
 		image.set_from_stock("gtk-media-play",gtk.ICON_SIZE_BUTTON)
 		self._play_pause_button.set_image(image)
-		#how to safely release XV when paused?  Right now if we release while
-		#paused we crash.  can only release when stopped
+		self.emit('paused')
 		
 	def stop(self):
 		#this should release the port, but I hate having a stop button on a computer
 		#because it doesn't make sense.
+		#unbreak: when video is stopped, we need this option to keep the window black
+		self._drawing_area.set_flags(gtk.DOUBLE_BUFFERED)
 		try: self._media_position = self._pipeline.query_position(gst.FORMAT_TIME)[0]
 		except: pass
 		self._pipeline.set_state(gst.STATE_READY)
@@ -336,6 +358,7 @@ class GStreamerPlayer(gobject.GObject):
 			self._v_sink.set_state(gst.STATE_NULL)
 			self._v_sink = self._get_video_sink(True)
 			self._pipeline.set_property('video-sink',self._v_sink)
+		self.emit('paused')
 			
 	def ff(self):
 		new_pos = self._media_position+15000000000L #15 seconds I think
@@ -367,10 +390,10 @@ class GStreamerPlayer(gobject.GObject):
 	def prev(self):
 		selection = self._queue_listview.get_selection()
 		self._pipeline.set_state(gst.STATE_READY)
+		self._current_index -= 1
 		if self._current_index <= 0:
 			self._current_index = 0
 			self.seek(0)
-		self._current_index -= 1
 		selection.unselect_all()
 		selection.select_path((self._current_index,))
 		self._seek_scale.set_range(0,1)
@@ -405,17 +428,17 @@ class GStreamerPlayer(gobject.GObject):
 	
 	def _on_prev_clicked(self, b): self.prev()
 	
-	def _on_type_discovered(self, discoverer, ismedia, filename, name):
+	def _on_type_discovered(self, discoverer, ismedia, filename, name, userdata):
 		if ismedia:
 			model = self._queue_listview.get_model()
 			uri = 'file://'+urllib.quote(filename)
 			if RUNNING_SUGAR:
 				name = '<span size="x-small">'+name+'</span>'
-			model.append([uri, name, ""])
-			self.emit('item-queued', filename, name)
+			model.append([uri, name, "", userdata])
+			self.emit('item-queued', filename, name, userdata)
 			self.save()
 		else:
-			self.emit('item-not-supported', filename, name)
+			self.emit('item-not-supported', filename, name, userdata)
 	
 	def _on_remove_clicked(self, b):
 		model, paths = self._queue_listview.get_selection().get_selected_rows()
@@ -425,7 +448,6 @@ class GStreamerPlayer(gobject.GObject):
 		iter_list = []
 		for path in paths:
 			if path[0] == self._current_index:
-				self.prev()
 				self.stop()
 			iter_list.append(model.get_iter(path))
 			
@@ -435,9 +457,20 @@ class GStreamerPlayer(gobject.GObject):
 		if len(model) == 0:
 			self._last_index = -1
 			self._current_index = 0
+			self._media_position = 0
+			self._media_duration = 0
+			self._update_time_label()
 		else:
-			self._current_index = [r[0] for r in model].index(current_uri)
-			self._last_index = self._current_index
+			try:
+				self._current_index = [r[0] for r in model].index(current_uri)
+				self._last_index = self._current_index
+			except ValueError:
+				# If the current_uri was removed, reset to top of list
+				self._current_index = 0
+				self._last_index = -1
+				self._media_position = 0
+				self._media_duration = 0
+				self._update_time_label()
 		self.emit('items-removed')
 		
 	def _on_seek_value_changed(self, widget):
@@ -477,15 +510,14 @@ class GStreamerPlayer(gobject.GObject):
 		#	self._x_overlay.expose()
 		self._v_sink.expose()
 		if not self.__is_exposed:
-			self.__is_exposed = True
 			model = self._queue_listview.get_model()
 			if len(model) > 0:
 				#self._prepare_display()
 				self._seek_to_saved_position()
+			self.__is_exposed = True
 				
 	###utility functions###
 	def _get_video_sink(self, compat=False):
-		print "getting new sink",compat
 		if compat:
 			sinks = ["ximagesink"]
 		else:
@@ -496,7 +528,6 @@ class GStreamerPlayer(gobject.GObject):
 				break
 			except:
 				print "couldn't init ",sink_str
-		print "default video sink:", sink_str
 		#according to totem this helps set things up (bacon-video-widget-gst-0.10:4290)
 		bus = self._pipeline.get_bus()
 		v_sink.set_bus(bus)
@@ -510,12 +541,10 @@ class GStreamerPlayer(gobject.GObject):
 		return True
 		
 	def _prepare_display(self, compat=False):
-		print "preparing display"
 		#if type(self._v_sink) != GstXVImageSink and not compat:
 		#do this right at some point: if we are using a substandard sink
 		#and we're not being specifically told to use it, try the better one
 		if 'gstximagesink' in str(type(self._v_sink)).lower() and not compat:
-			print "was using compat sink.  Try getting the better one"
 			self._v_sink = self._get_video_sink()
 			self._pipeline.set_property('video-sink',self._v_sink)
 		if compat:
@@ -543,6 +572,10 @@ class GStreamerPlayer(gobject.GObject):
 		#get video width and height so we can resize the pane
 		#see totem
 		#if (!(caps = gst_pad_get_negotiated_caps (pad)))
+		
+		#unbreakme: if there's no video, it doesn't draw right here either
+		self._drawing_area.set_flags(gtk.DOUBLE_BUFFERED)
+		
 		min_width = 200
 		max_width = self._hpaned.get_allocation().width - 200 #-200 for the list box
 		
@@ -557,6 +590,10 @@ class GStreamerPlayer(gobject.GObject):
  		if caps is None: #no big deal, this might be audio only
  			self._hpaned.set_position(max_width / 2)
  			return
+ 		
+ 		#unbreakme: without this option the video doesn't redraw correctly when exposed
+		self._drawing_area.unset_flags(gtk.DOUBLE_BUFFERED)	
+ 			
   		s = caps[0]
   		movie_aspect = float(s['width']) / s['height']
   		display_height = self._drawing_area.get_allocation().height
@@ -605,8 +642,15 @@ class GStreamerPlayer(gobject.GObject):
 		self.__no_seek = True
 		self._update_seek_bar()
 		self._update_time_label()
+		if self._prepare_save:
+			self._prepare_save = False
+			self.save()
 		self.__no_seek = False
 		return self._pipeline.get_state()[1] == gst.STATE_PLAYING
+		
+	def _periodic_save_cb(self):
+		self._prepare_save = True
+		return True
 
 	def _update_seek_bar(self):
 		try:
@@ -671,9 +715,7 @@ class GStreamerPlayer(gobject.GObject):
 					for row in model:
 						i+=1
 						if playing_uri == row[0]:
-							print "moving current file from", paths_to_copy[0][0],
 							self._last_index = self._current_index = i
-							print "to",i
 							row[2]="&#8226;" #bullet
 						else:
 							row[2]=""							
@@ -730,10 +772,9 @@ class GStreamerErrorDialog(gtk.Window):
 			
 	def _on_delete_event(self, widget, event):
 		return self.hide_on_delete()
-	
+		
 #########app
 def do_quit(self, widget, player):
-	print "finish"
 	player.finish()
 	gtk.main_quit()
 	
