@@ -4,8 +4,7 @@ import gobject
 import sys, os, os.path
 import sets
 
-import traceback
-
+import code
 
 import ptvDB
 import penguintv
@@ -38,7 +37,8 @@ N_FEEDS     = 0
 N_PLAYER    = 1
 N_DOWNLOADS = 2
 
-import EditTagsMultiDialog
+#import EditTagsMultiDialog
+import TagEditorNG
 import AddSearchTagDialog
 import EditSearchesDialog
 import FeedFilterDialog
@@ -47,16 +47,27 @@ import FeedFilterPropertiesDialog
 import SynchronizeDialog
 import FilterSelectorDialog
 import MainWindow, FeedList, EntryList, EntryView, PlanetView, DownloadView
+
 if utils.HAS_GSTREAMER:
 	import GStreamerPlayer
 
-class MainWindow:
+class MainWindow(gobject.GObject):
 	COLUMN_TITLE = 0
 	COLUMN_ITEM = 1
 	COLUMN_BOLD = 2
 	COLUMN_STICKY_FLAG = 3
+	
+	__gsignals__ = {
+		'player-show': (gobject.SIGNAL_RUN_FIRST, 
+						   gobject.TYPE_NONE, 
+						   ([])),
+		'player-hide': (gobject.SIGNAL_RUN_FIRST, 
+						   gobject.TYPE_NONE, 
+						   ([]))
+	}
 
-	def __init__(self, app, glade_prefix, use_internal_player=False, window=None):
+	def __init__(self, app, glade_prefix, use_internal_player=False, window=None, status_icon=None):
+		gobject.GObject.__init__(self)
 		self._app = app
 		self._app_inited = False
 		self._mm = self._app.mediamanager
@@ -74,27 +85,35 @@ class MainWindow:
 		self._use_internal_player = False
 		if utils.HAS_GSTREAMER and use_internal_player:
 			self._use_internal_player = True
+			
+		self._status_icon = status_icon
 		
 		self._active_filter_name = FeedList.BUILTIN_TAGS[FeedList.ALL]
 		self._active_filter_index = FeedList.ALL
 		
 		##other WINDOWS we open
-		self._window_add_search = AddSearchTagDialog.AddSearchTagDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_add_search_tag",'penguintv'),self._app)
-		
+		if utils.HAS_LUCENE:
+			self._window_add_search = AddSearchTagDialog.AddSearchTagDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_add_search_tag",'penguintv'),self._app)
+			self._feed_filter_properties_dialog = FeedFilterPropertiesDialog.FeedFilterPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_filter_properties",'penguintv'),self._app)
+		if not utils.RUNNING_SUGAR:
+			self._sync_dialog = SynchronizeDialog.SynchronizeDialog(os.path.join(self._glade_prefix,'penguintv.glade'), self._db)
+			
 		self._feed_properties_dialog = FeedPropertiesDialog.FeedPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_feed_properties",'penguintv'),self._app)
-		self._feed_filter_properties_dialog = FeedFilterPropertiesDialog.FeedFilterPropertiesDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "window_filter_properties",'penguintv'),self._app)
-		self._sync_dialog = SynchronizeDialog.SynchronizeDialog(os.path.join(self._glade_prefix,'penguintv.glade'), self._db)
+		
 		self._filter_selector_dialog = FilterSelectorDialog.FilterSelectorDialog(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "dialog_tag_favorites",'penguintv'),self)
 		
 		#signals
 		self._app.connect('feed-added', self.__feed_added_cb)
 		self._app.connect('feed-removed', self.__feed_removed_cb)
 		self._app.connect('feed-polled', self.__feed_polled_cb)
+		self._app.connect('download-finished', self.__download_finished_cb)
+		self._app.connect('setting-changed', self.__setting_changed_cb)
+		self._app.connect('tags-changed', self.__tags_changed_cb)
 	
 		#most of the initialization is done on Show()
-		if window:
-			window.connect('show', self.Show)
-		
+		if utils.RUNNING_SUGAR:
+			gobject.idle_add(self.Show, window)
+			
 	def __link_activated_cb(self, o, link):
 		self._app.activate_link(link)
 				
@@ -108,20 +127,37 @@ class MainWindow:
 				
 	def __feed_added_cb(self, app, feed_id, success):
 		if success:
-			self.select_feed(feed_id)
+			#HACK: we know it will already be selected
+			#self.select_feed(feed_id)
 			self.display_status_message(_("Feed Added"))
 			gobject.timeout_add(2000, self.display_status_message, "")
 		else:
 			self.display_status_message(_("Error adding feed"))
 			self.select_feed(feed_id)
 			
-	def __feed_polled_cb(self, app, feed_id):
-		self.display_status_message(_("Feed Updated"))
-		gobject.timeout_add(2000, self.display_status_message, "")
+	def __feed_polled_cb(self, app, feed_id, update_data):
+		if not update_data.has_key('polling_multiple'):
+			self.display_status_message(_("Feed Updated"))
+			gobject.timeout_add(2000, self.display_status_message, "")
 			
 	def __feed_removed_cb(self, app, feed_id):
 		self.update_filters()
-
+		
+	def __download_finished_cb(self, app, d):
+		self._download_view.update_downloads()
+		
+	def __setting_changed_cb(self, app, typ, datum, value):
+		if datum == '/apps/penguintv/show_notifications':
+			show_notifs_item = self._widgetTree.get_widget('show_notifications')
+			if show_notifs_item.get_active() != value:
+				show_notifs_item.set_active(value)
+				
+	def __tags_changed_cb(self, app, val):
+		self.update_filters()
+		
+	def update_downloads(self):
+		self._download_view.update_downloads()
+		
 #	def __getitem__(self, key):
 #		return self.widgets.get_widget(key)
 
@@ -139,18 +175,18 @@ class MainWindow:
 		application window"""
 		#sys.stderr.write("show,"+str(dock_widget))
 		
-		if not EntryView.MOZ_OK and self.layout == "planet":
+		if not utils.HAS_MOZILLA and self.layout == "planet":
 			print "requested planet layout, but can't use because gtkmozembed isn't installed correctly (won't import)"
 			self.layout = "standard"
 		
-		if dock_widget is None:  #if we are loading in a regular window...
+		if not utils.RUNNING_SUGAR:  #if we are loading in a regular window...
 			self._load_app_window()
 			if not utils.HAS_LUCENE:
 				#remove UI elements that don't apply without search
 				self._widgetTree.get_widget('saved_searches').hide()
 				self._widgetTree.get_widget('separator11').hide()
 				self._widgetTree.get_widget('reindex_searches').hide()
-			if not EntryView.MOZ_OK:
+			if not utils.HAS_MOZILLA:
 				self._widgetTree.get_widget('planet_layout').hide()
 			self._window = self.app_window
 		else:  #if we are in OLPC mode and just have to supply a widget...
@@ -159,16 +195,22 @@ class MainWindow:
 			self.app_window = None
 
 			vbox = gtk.VBox()
-			
-			vbox.pack_start(self._load_toolbar(), False, False)
+			self._widgetTree = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', 'toolbar_holder','penguintv')
+			toolbar = self._load_toolbar()
+			toolbar.unparent()
+			vbox.pack_start(toolbar, False, False)
 			self._layout_dock = self.load_notebook()
 			self._layout_dock.add(self.load_layout())
 			vbox.pack_start(self._notebook)
 			self._status_view = MainWindow._my_status_view()
 			vbox.pack_start(self._status_view, False, False)
-			dock_widget.add(vbox)
+			dock_widget.set_canvas(vbox)
 			dock_widget.show_all()
-			self._widgetTree.get_widget('debug_exit_toolbutton').hide()
+			
+			for key in dir(self.__class__): #python insaneness
+				if key[:3] == 'on_':
+					self._widgetTree.signal_connect(key, getattr(self, key))
+			
 			self._window = dock_widget
 		self._notebook.show_only(N_FEEDS)
 		if not utils.HAS_LUCENE:
@@ -178,17 +220,15 @@ class MainWindow:
 		if self._use_internal_player:
 			if self._gstreamer_player.get_queue_count() > 0:
 				self._notebook.show_page(N_PLAYER)
+				self.emit('player-show')
 				
 		if not self._app_inited:
 			gobject.idle_add(self._app.post_show_init)
 			self._app_inited = True
-				
+			
+		return False
+			
 	def _load_toolbar(self):
-		if self._widgetTree is None:
-			self._widgetTree = gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), 'toolbar1','penguintv')
-			for key in dir(self.__class__): #python insaneness
-				if key[:3] == 'on_':
-					self._widgetTree.signal_connect(key, getattr(self, key))
 		toolbar = self._widgetTree.get_widget('toolbar1')
 		
 		#set up separator (see below)
@@ -197,37 +237,8 @@ class MainWindow:
 		vseparator_toolitem.set_expand(True)
 		vseparator.set_draw(False)
 		
-		if utils.RUNNING_SUGAR:
-			vsep2 = self._widgetTree.get_widget('vseparator2')
-			vseparator.set_property('visible',True)
-			pref_button = self._widgetTree.get_widget('preferences_toolbutton')
-			pref_button.set_property('visible',True)
-			pref_button.set_property('label',_("Preferences"))
-			
-			debug_exit_button = self._widgetTree.get_widget('debug_exit_toolbutton')
-			debug_exit_button.set_property('visible',False)
-			#debug_exit_button.set_property('label',_("Exit (DEBUG)"))
-			
-			theme = gtk.icon_theme_get_default()
-			theme.append_search_path(os.path.join(self._glade_prefix, "icons"))
-			self._widgetTree.get_widget('feed_add_button').set_stock_id(None)
-			self._widgetTree.get_widget('feed_add_button').set_icon_name('stock-add')
-			self._widgetTree.get_widget('feed_remove').set_stock_id(None)
-			self._widgetTree.get_widget('feed_remove').set_icon_name('stock-remove')
-			self._widgetTree.get_widget('feeds_poll').set_stock_id(None)
-			self._widgetTree.get_widget('feeds_poll').set_icon_name('stock-continue')
-			self._widgetTree.get_widget('download_unviewed').set_stock_id(None)
-			self._widgetTree.get_widget('download_unviewed').set_icon_name('stock-go-down')
-			self._widgetTree.get_widget('play_unviewed').set_stock_id(None)
-			self._widgetTree.get_widget('play_unviewed').set_icon_name('stock-media-play')
-			self._widgetTree.get_widget('preferences_toolbutton').set_stock_id(None)
-			self._widgetTree.get_widget('preferences_toolbutton').set_icon_name('stock-preferences')
 		self._disk_usage_widget = self._widgetTree.get_widget('disk_usage')
 		self._disk_usage_widget.set_use_markup(True)
-		if utils.RUNNING_SUGAR:
-			l = self._widgetTree.get_widget('using_label')
-			l.set_use_markup(True)
-			l.set_markup(_('<span color="#FFFFFF">Using: </span>'))
 	
 		return toolbar
 		
@@ -253,34 +264,32 @@ class MainWindow:
 			
 	def _load_app_window(self):
 		self._widgetTree = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', 'app','penguintv')
+		
 		notebook_dock = self._widgetTree.get_widget('layout_dock')
 		self.app_window = self._widgetTree.get_widget('app')
-		
 		fancy_feedlist_item = self._widgetTree.get_widget('fancy_feed_display')
-		fancy_feedlist_item.set_active(self._db.get_setting(ptvDB.BOOL, '/apps/penguintv/fancy_feedlist', True))
+		fancy_feedlist_item.set_active(self._db.get_setting(ptvDB.BOOL, 
+		                               '/apps/penguintv/fancy_feedlist', True))
+		show_notifs_item = self._widgetTree.get_widget('show_notifications')
+		show_notifs_item.set_active(self._db.get_setting(ptvDB.BOOL, 
+		                           '/apps/penguintv/show_notifications', True))
 		self._widgetTree.get_widget(self.layout+"_layout").set_active(True)
-		
-		try:
-			self.app_window.set_icon_from_file(utils.GetPrefix()+"/share/pixmaps/penguintvicon.png")
-		except:
-			try:
-				self.app_window.set_icon_from_file(utils.GetPrefix()+"/share/penguintvicon.png") #in case the install is still in the source dirs
-			except:
-				self.app_window.set_icon_from_file(self._glade_prefix+"/penguintvicon.png")
+		self.app_window.set_icon_from_file(utils.get_icon_filename())
 		self._status_view = self._widgetTree.get_widget("appbar")
-		
 		self._load_toolbar()
 		
 		#load the layout
 		self._layout_dock = self.load_notebook()
 		notebook_dock.add(self._notebook)
 		self._layout_dock.add(self.load_layout())
+
 		self.app_window.show_all()
-		
-				
+
 		#final setup for the window comes from gconf
 		x = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_position_x', 40)
 		y = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_position_y', 40)
+		if x < 0: x = 0
+		if y < 0: y = 0
 		self.app_window.move(x,y)
 		w = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_size_x', 500)
 		h = self._db.get_setting(ptvDB.INT, '/apps/penguintv/app_window_size_y', 500)
@@ -309,6 +318,7 @@ class MainWindow:
 			self._gstreamer_player.connect('item-queued', self._on_player_item_queued)
 			self._gstreamer_player.connect('items-removed', self._on_player_items_removed)
 			self._gstreamer_player.Show()
+			self.emit('player-show')
 		self._player_label = gtk.Label('<span size="small">'+_('Player')+'</span>')
 		self._player_label.set_property('use-markup',True)
 		self._notebook.append_page(p_vbox, self._player_label)
@@ -336,7 +346,8 @@ class MainWindow:
 		self._notebook.set_current_page(page)
 		
 	def load_layout(self):
-		components = gtk.glade.XML(self._glade_prefix+'/penguintv.glade', self.layout+'_layout_container','penguintv') #MAGIC
+		components = gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), 
+		                           self.layout+'_layout_container','penguintv') #MAGIC
 		self._layout_container = components.get_widget(self.layout+'_layout_container')
 		#dock_widget.add(self._layout_container)
 		
@@ -345,55 +356,29 @@ class MainWindow:
 			fancy = False
 		
 		self.feed_list_view = FeedList.FeedList(components,self._app, self._db, fancy)
-		renderer_str = self._db.get_setting(ptvDB.STRING, '/apps/penguintv/renderrer', "GTKHTML") #stupid misspelling (renderrer != renderer)
-		
-		if renderer_str == "GTKHTML":
-			renderer = EntryView.GTKHTML
-		elif renderer_str == "DEMOCRACY_MOZ":
-			renderer = EntryView.DEMOCRACY_MOZ
-		elif renderer_str == "MOZILLA":
+		if utils.HAS_MOZILLA:
 			renderer = EntryView.MOZILLA
 		else:
+			print "WARNING: gtkmozembed not found, falling back on gtkhtml. PlanetView disabled"
 			renderer = EntryView.GTKHTML
-		if utils.RUNNING_SUGAR:
-			renderer = EntryView.MOZILLA
-			
-		if not EntryView.MOZ_OK:
-			renderer = EntryView.GTKHTML
-					
-		def load_renderer(x,recur=0):
-			"""little function I define so I can recur"""
-			if recur==2:
-				print "too many tries"
-				self._app.do_quit()
-				sys.exit(2)
-			#try:
-			if self.layout != "planet":
-				self.entry_view = EntryView.EntryView(components, self.feed_list_view, 
-									 				   self.entry_list_view, self._app, self, x)
-			else:
-				self.entry_view = PlanetView.PlanetView(components, self.feed_list_view, 
-								                        self._app, self, self._db, x)
-			#except Exception, e:
-			#	print e
-			#	if renderer == EntryView.DEMOCRACY_MOZ:
-			#		if  _FORCE_DEMOCRACY_MOZ:
-			#			load_renderer(EntryView.DEMOCRACY_MOZ,recur+1)
-			#		else:
-			#			print "Error instantiating Democracy Mozilla renderer, falling back to GTKHTML"
-			#			print "(if running from source dir, build setup.py and copy MozillaBrowser.so to democracy_moz/)"
-			#			load_renderer(EntryView.GTKHTML,recur+1)
-			#	else:
-			#		print "Error loading renderer"
-			#		self._app.do_quit()
+		
+		if self.layout == "planet" and renderer != EntryView.MOZILLA:
+			self.layout = "standard"
+			return self.load_layout()	
 		
 		if self.layout != "planet":
-			self.entry_list_view = EntryList.EntryList(components, self._app, self.feed_list_view, self, self._db)
-			load_renderer(renderer)
+			self.entry_list_view = EntryList.EntryList(components, self._app, 
+			                                           self.feed_list_view, self, self._db)
+			self.entry_view = EntryView.EntryView(components, self.feed_list_view, 
+										          self.entry_list_view, self._app, self, renderer)
 		else:
-			load_renderer(renderer)
+			self.entry_view = PlanetView.PlanetView(components, self.feed_list_view, 
+										            self._app, self, self._db, renderer)
 			self.entry_list_view = self.entry_view
 			
+		if renderer == EntryView.GTKHTML:
+			self._widgetTree.get_widget('planet_layout').hide()	
+			
 		for key in dir(self.__class__): #python insaneness
 			if key[:3] == 'on_':
 				components.signal_connect(key, getattr(self, key))
@@ -417,6 +402,7 @@ class MainWindow:
 			self.entry_pane = self.feed_pane #cheat
 		
 		self._filter_container = components.get_widget('filter_container')
+		self._filter_unread_checkbox = components.get_widget('unread_filter')
 		self._filter_selector_button = components.get_widget('filter_selector_button')
 		#self._filter_menubar = components.get_widget('filter_menubar')
 		#self._filter_menuitem = gtk.MenuItem()#components.get_widget('filter_menu_root')
@@ -464,11 +450,13 @@ class MainWindow:
 			try:
 				filter_index = [row[F_NAME] for row in self._filters].index(val)
 				cur_filter = self._filters[filter_index]
-				if cur_filter[F_TYPE] != ptvDB.T_SEARCH and filter_index!=FeedList.SEARCH:
-					#self.feed_list_view.set_filter(filter_index,val)
-					self.set_active_filter(filter_index)
+				if utils.HAS_LUCENE:
+					if cur_filter[F_TYPE] == ptvDB.T_SEARCH or filter_index==FeedList.SEARCH:
+						self.set_active_filter(FeedList.ALL)
+					else:
+						self.set_active_filter(filter_index)
 				else:
-					self.set_active_filter(FeedList.ALL)
+					self.set_active_filter(filter_index)
 			except ValueError: #didn't find the item in the model (.index(val) fails)
 				self.set_active_filter(FeedList.ALL)
 		else:
@@ -494,10 +482,6 @@ class MainWindow:
 		del self._disk_usage_widget
 		
 	def get_parent(self):
-		#if self.app_window is not None:
-		#	return self.app_window
-		#else:
-		#	return self._layout_dock.get_parent_window()
 		return self._window
 		
 	def toggle_fullscreen(self, fullscreen):
@@ -634,9 +618,8 @@ class MainWindow:
 			
 	def on_edit_tags_for_all_activate(self, event):
 		"""Bring up mass tag creation window"""
-		window_edit_tags_multi = EditTagsMultiDialog.EditTagsMultiDialog(gtk.glade.XML(self._glade_prefix+'/penguintv.glade', "window_edit_tags_multi",'penguintv'),self._app)
+		window_edit_tags_multi = TagEditorNG.TagEditorNG(gtk.glade.XML(os.path.join(self._glade_prefix,'penguintv.glade'), "dialog_tag_editor_ng",'penguintv'), self._app)
 		window_edit_tags_multi.show()
-		window_edit_tags_multi.set_feed_list(self._db.get_feedlist())
 			
 	def on_export_opml_activate(self, event):
 		self._app.export_opml()
@@ -658,7 +641,7 @@ class MainWindow:
 					url = url+c
 			if url.split(':')[0] == 'feed': #stupid wordpress does 'feed:http://url.com/whatever'
 				url = url[url.find(':')+1:]
-			self._app.add_feed(url)
+			self._app.add_feed(url, url)
 			
 	def on_feedlistview_button_press_event(self, widget, event):          
 		if event.button==3: #right click     
@@ -784,9 +767,9 @@ class MainWindow:
 		
 	def on_import_opml_activate(self, event):
 		dialog = gtk.FileChooserDialog(_('Select OPML...'),None, action=gtk.FILE_CHOOSER_ACTION_OPEN,
-                                  buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
+								  buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
 		dialog.set_default_response(gtk.RESPONSE_OK)
-    
+	
 		filter = gtk.FileFilter()
 		filter.set_name("OPML files")
 		filter.add_pattern("*.opml")
@@ -817,8 +800,8 @@ class MainWindow:
 				self.toggle_fullscreen(self._fullscreen)
 		
 	def on_mark_entry_as_viewed_activate(self,event):
-		entry = self.entry_list_view.get_selected()['entry_id']
-		self._app.mark_entry_as_viewed(entry)
+		entry = self.entry_list_view.get_selected()
+		self._app.mark_entry_as_viewed(entry['entry_id'], entry['feed_id'])
 		
 	def on_mark_entry_as_unviewed_activate(self,event):
 		entry = self.entry_list_view.get_selected()['entry_id']
@@ -840,6 +823,7 @@ class MainWindow:
 			self._gstreamer_player.load()
 			if self._gstreamer_player.get_queue_count() > 0:
 				self._notebook.show_page(N_PLAYER)
+				self.emit('player-show')
 		
 	#def _on_gst_player_realized(self, widget):
 	#	print "seek seek seek"
@@ -855,8 +839,9 @@ class MainWindow:
 	def on_play_unviewed_clicked(self, event):
 		self._app.play_unviewed()
 		
-	def _on_player_item_queued(self, player, filename, name):
-		self._notebook.show_page(N_PLAYER)	
+	def _on_player_item_queued(self, player, filename, name, userdata):
+		self._notebook.show_page(N_PLAYER)
+		self.emit('player-show')	
 		#if player.get_queue_count() == 1:
 		#	try:
 		#		self._notebook.set_current_page(N_PLAYER)
@@ -868,6 +853,7 @@ class MainWindow:
 	def _on_player_items_removed(self, player):
 		if player.get_queue_count() == 0:
 			self._notebook.hide_page(N_PLAYER)
+			self.emit('player-hide')
 			player.stop()
 		self._player_label.set_markup(_('<span size="small">Player (%d)</span>') % player.get_queue_count())
 		
@@ -973,6 +959,10 @@ class MainWindow:
 	def on_fancy_feed_display_activate(self, menuitem):
 		self.feed_list_view.set_fancy(menuitem.get_active())
 		self._db.set_setting(ptvDB.BOOL, '/apps/penguintv/fancy_feedlist', menuitem.get_active())
+		
+	def on_show_notifications_activate(self, menuitem):
+		self._db.set_setting(ptvDB.BOOL, '/apps/penguintv/show_notifications',
+							 menuitem.get_active())
 				
 	def activate_layout(self, layout):
 		"""gets called by app when it's ready"""
@@ -991,11 +981,15 @@ class MainWindow:
 		if self._use_internal_player:
 			if self._gstreamer_player.get_queue_count() > 0:
 				self._notebook.show_page(N_PLAYER)
+				self.emit('player-show')
 		#can't reset changing_layout because app hasn't updated pointers yet
 		
 	def is_changing_layout(self):
 		return self.changing_layout
 		
+	def on_unread_filter_toggled(self, event): 	 
+		self.feed_list_view.set_unread_toggle(self._filter_unread_checkbox.get_active()) 	 
+		
 	def display_status_message(self, m, update_category=U_STANDARD):
 		"""displays a status message on the main status bar.  If this is a polling update or download
 		   update, we don't overwrite what's there."""	
@@ -1009,9 +1003,13 @@ class MainWindow:
 		if current_text == "":
 			self._status_owner = update_category
 			self._status_view.set_status(m)
+			if utils.HAS_STATUS_ICON:
+				self._status_icon.set_tooltip(m)
 		else:
 			if update_category >= self._status_owner:
 				self._status_view.set_status(m)
+				if utils.HAS_STATUS_ICON:
+					self._status_icon.set_tooltip(m)
 				if m == "":
 					self._status_owner = U_NOBODY
 				else:
@@ -1044,14 +1042,18 @@ class MainWindow:
 		if self._state == S_MANUAL_SEARCH:
 			self.search_entry.set_text("")
 		if self._state == S_LOADING_FEEDS:
-			self._widgetTree.get_widget("feed_add_button").set_sensitive(True)
-			self._widgetTree.get_widget("feed_remove").set_sensitive(True)
 			if not utils.RUNNING_SUGAR:
+				self._widgetTree.get_widget("feed_add_button").set_sensitive(True)
+				self._widgetTree.get_widget("feed_remove").set_sensitive(True)
 				#these are menu items
 				self._widgetTree.get_widget("add_feed").set_sensitive(True)
 				self._widgetTree.get_widget("add_feed_filter").set_sensitive(True)
 				self._widgetTree.get_widget("remove_feed").set_sensitive(True)
 				self._widgetTree.get_widget("properties").set_sensitive(True)
+			else:
+				self._widgetTree.get_widget("feed_add_button").set_sensitive(True)
+				self._widgetTree.get_widget("feed_remove").set_sensitive(True)
+			
 			self.display_status_message("")	
 			self.update_progress_bar(-1,U_LOADING)
 			
@@ -1074,15 +1076,17 @@ class MainWindow:
 			self.search_entry.set_text("")
 		
 		if new_state == S_LOADING_FEEDS:
-			self._widgetTree.get_widget("feed_add_button").set_sensitive(False)
-			self._widgetTree.get_widget("feed_remove").set_sensitive(False)
-			
 			if not utils.RUNNING_SUGAR: 
+				self._widgetTree.get_widget("feed_add_button").set_sensitive(False)
+				self._widgetTree.get_widget("feed_remove").set_sensitive(False)
 				#these are menu items
 				self._widgetTree.get_widget("add_feed").set_sensitive(False)
 				self._widgetTree.get_widget("add_feed_filter").set_sensitive(False)
 				self._widgetTree.get_widget("remove_feed").set_sensitive(False)
 				self._widgetTree.get_widget("properties").set_sensitive(False)
+			else:
+				self._widgetTree.get_widget("feed_add_button").set_sensitive(False)
+				self._widgetTree.get_widget("feed_remove").set_sensitive(False)
 			
 		self._state = new_state
 
@@ -1124,7 +1128,7 @@ class MainWindow:
 					if favorite > 0:
 						self._favorite_filters.append([favorite, tag,tag, i]) 
 		
-		tags = self._db.get_all_tags(ptvDB.T_TAG)	
+		tags = self._db.get_all_tags(ptvDB.T_TAG)
 		if tags:
 			sep = gtk.SeparatorMenuItem()
 			self._filter_menu.append(sep)
@@ -1164,7 +1168,7 @@ class MainWindow:
 			
 		#sep = gtk.SeparatorMenuItem()
 		#self._filter_menu.append(sep)
-	
+		
 		menuitem = gtk.MenuItem(_('Edit Favorite Tags...'))
 		menuitem.connect('activate', self.on_edit_favorite_tags)
 		self._filter_menu.append(menuitem) 
@@ -1251,10 +1255,7 @@ class MainWindow:
 	def update_disk_usage(self, size):
 		if self._disk_usage_widget is None:
 			return
-		if utils.RUNNING_SUGAR:
-			self._disk_usage_widget.set_markup('<span color="#FFFFFF">'+utils.format_size(size)+"</span>")
-		else:
-			self._disk_usage_widget.set_markup(utils.format_size(size))
+		self._disk_usage_widget.set_markup(utils.format_size(size))
 
 	def update_download_progress(self):
 		progresses = self._mm.get_download_list(Downloader.DOWNLOADING)
@@ -1293,9 +1294,6 @@ class MainWindow:
 		self._download_view.update_downloads()
 		self._update_notebook_tabs(len(progresses)+len(queued)+len(paused))
 		
-	def download_finished(self): #FIXME: convert to gobject signal one day
-		self._download_view.update_downloads()
-		
 	def _update_notebook_tabs(self, number):
 		if number == 0:
 			self._notebook.hide_page(N_DOWNLOADS)
