@@ -5,24 +5,22 @@
 #OH NOES!
 
 
-import socket
-import SocketServer
-import SimpleHTTPServer
-import urllib
-import threading
-import random
 import logging
 
 import gobject
-
-try:
-	import gtkmozembed
-except:
-	pass
+import gtk
 
 from EntryView import *
 import ptvDB
 import utils
+
+if not utils.RUNNING_SUGAR:
+	try:
+		import gtkmozembed
+	except:
+		pass
+else:
+	import sugar.browser
 	
 ENTRIES_PER_PAGE = 10
 
@@ -100,55 +98,65 @@ class PlanetView(gobject.GObject):
                 style.base[gtk.STATE_INSENSITIVE].green / 256)
 		
 		if self._renderer == GTKHTML:
-			logging.error("not supported (need AJAX, believe it or not)")
-			print "not supported (need AJAX, believe it or not)"
+			logging.error("gtkhtml and planetview not supported")
+			print "gtkhtml and planetview not supported"
 			return
+			#self._USING_AJAX = False
 		elif self._renderer == MOZILLA:
 			if utils.RUNNING_SUGAR:
+				self._USING_AJAX = False
 				f = open (os.path.join(self._app.glade_prefix,"mozilla-planet-olpc.css"))
 				for l in f.readlines(): self._css += l
 				f.close()
 				
-				import _sugar
-				_sugar.browser_startup(self._db.home, 'gecko')
-				self._moz = _sugar.Browser()
+				self._moz = sugar.browser.Browser()
 			else:
+				self._USING_AJAX = True
 				f = open (os.path.join(self._app.glade_prefix,"mozilla-planet.css"))
 				for l in f.readlines(): self._css += l
 				f.close()
+				if not utils.init_gtkmozembed():
+					print "Error initializing mozilla.  Penguintv may crash shortly"
 				gtkmozembed.set_profile_path(self._db.home, 'gecko')
 				gtkmozembed.push_startup()
 				self._moz = gtkmozembed.MozEmbed()
 				
-			
 			self._moz.connect("open-uri", self._moz_link_clicked)
+			self._moz.connect("new-window", self._moz_new_window)
 			self._moz.connect("link-message", self._moz_link_message)
 			self._moz.connect("realize", self._moz_realize, True)
 			self._moz.connect("unrealize", self._moz_realize, False)
 			self._moz.load_url("about:blank")
 			scrolled_window.add_with_viewport(self._moz)
-			#scrolled_window.add(self._moz)
 			self._moz.show()
 			if utils.HAS_GCONF:
 				import gconf
 				self._conf = gconf.client_get_default()
 				self._conf.notify_add('/desktop/gnome/interface/font_name',self._gconf_reset_moz_font)
 			self._reset_moz_font()
+		self.display_item()
 		html_dock.show_all()
-		while True:
-			try:
-				if PlanetView.PORT == 8050:
+		if self._USING_AJAX:
+			print "initializing ajax server"
+			import threading
+			from ajax import EntryInfoServer, MyTCPServer
+
+			while True:
+				try:
+					if PlanetView.PORT == 8050:
+						break
+					self._update_server = MyTCPServer.MyTCPServer(('', PlanetView.PORT), EntryInfoServer.EntryInfoServer)
 					break
-				self._update_server = PlanetView.MyTCPServer(('', PlanetView.PORT), PlanetView.EntryInfoServer)
-				break
-			except:
-				PlanetView.PORT += 1
-		if PlanetView.PORT==8050:
-			logging.warning("tried a lot of ports without success.  Problem?")
-			print "tried a lot of ports without success.  Problem?"
-		t = threading.Thread(None, self._update_server.serve_forever)
-		t.setDaemon(True)
-		t.start()
+				except:
+					PlanetView.PORT += 1
+			if PlanetView.PORT==8050:
+				logging.warning("tried a lot of ports without success.  Problem?")
+				print "tried a lot of ports without success.  Problem?"
+			t = threading.Thread(None, self._update_server.serve_forever)
+			t.setDaemon(True)
+			t.start()
+		else:
+			print "not using ajax"
 		
 		#signals
 		self._handlers = []
@@ -164,6 +172,9 @@ class PlanetView(gobject.GObject):
 		self._handlers.append((self._app.disconnect, h_id))
 		h_id = self._app.connect('entry-updated', self.__entry_updated_cb)
 		self._handlers.append((self._app.disconnect, h_id))
+		screen = gtk.gdk.screen_get_default()
+		h_id = screen.connect('size-changed', self.__size_changed_cb)
+		self._handlers.append((screen.disconnect, h_id))
 		
 	def __feedlist_feed_selected_cb(self, o, feed_id):
 		self.populate_entries(feed_id)
@@ -175,7 +186,7 @@ class PlanetView(gobject.GObject):
 		if success:
 			self.populate_entries(feed_id)
 			
-	def __feed_polled_cb(self, app, feed_id):
+	def __feed_polled_cb(self, app, feed_id, update_data):
 		if feed_id == self._current_feed_id:
 			self.populate_entries(feed_id)
 			
@@ -185,7 +196,11 @@ class PlanetView(gobject.GObject):
 	def __entry_updated_cb(self, app, entry_id, feed_id):
 		self.update_entry_list(entry_id)
 		if feed_id == self._current_feed_id:
-			self.populate_entries(feed_id)
+			self._render_entries()
+			
+	def __size_changed_cb(self, screen):
+		"""Redraw after xrandr calls"""
+		self._render_entries()
 		
 	#entrylist functions
 	def populate_if_selected(self, feed_id):
@@ -201,7 +216,13 @@ class PlanetView(gobject.GObject):
 			self.clear_entries()
 			return
 			
-		db_entrylist = self._db.get_entrylist(feed_id)
+		try:
+			db_entrylist = self._db.get_entrylist(feed_id)
+		except ptvDB.NoEntry, e:
+			print "error displaying search"
+			self._render(_("There was an error displaying the search results.  Please reindex searches and try again"))
+			return
+
 		if feed_id != self._current_feed_id:
 			self._current_feed_id = feed_id
 			self._first_entry = 0
@@ -211,11 +232,15 @@ class PlanetView(gobject.GObject):
 				self._auth_info = (feed_id,feed_info['auth_userpass'], feed_info['auth_domain'])
 			else:
 				self._auth_info = (-1, "","")
-			self._update_server.clear_updates()
+			if self._USING_AJAX:
+				self._update_server.clear_updates()
 		#always update title in case it changed... it's a cheap lookup
 		self._feed_title = self._db.get_feed_title(feed_id)
-		self._entrylist = [e[0] for e in db_entrylist]
-		self._readinfo  = [e[3] for e in db_entrylist]
+		self._entrylist = []
+		self._readinfo = []
+		for e in db_entrylist:
+			self._entrylist.append(e[0])
+			self._readinfo.append(e[3])
 		self._render_entries()
 		
 	def auto_pane(self):
@@ -242,7 +267,7 @@ class PlanetView(gobject.GObject):
 			self._render_entries(query)
 		except ptvDB.NoEntry:
 			print "error displaying search"
-			self.display_custom_entry(_("There was an error displaying the search results.  Please reindex searches and try again"))
+			self._render(_("There was an error displaying the search results.  Please reindex searches and try again"))
 		
 	def unshow_search(self):
 		self._render("<html><body></body></html")
@@ -252,12 +277,14 @@ class PlanetView(gobject.GObject):
 		pass
 		
 	def clear_entries(self):
+		self._current_feed_id = -1
 		self._first_entry = 0
 		self._entry_store={}
 		self._entrylist = []
 		self._readinfo  = None
 		self._render("<html><body></body></html")
-		self._update_server.clear_updates()
+		if self._USING_AJAX:
+			self._update_server.clear_updates()
 		
 	def _unset_state(self):
 		self.clear_entries()
@@ -285,14 +312,11 @@ class PlanetView(gobject.GObject):
 		if self._custom_message == message:
 			return
 		self._custom_message = message
-		self.populate_entries()
 		
 	def undisplay_custom_entry(self):
 		if self._custom_message == "":
 			return
 		self._custom_message = ""
-		#print "custom: blank (undisplay)"
-		self.populate_entries()
 		
 	def display_item(self, item=None, highlight=""):
 		if item is None:
@@ -306,12 +330,15 @@ class PlanetView(gobject.GObject):
 	def finish(self):
 		for disconnector, h_id in self._handlers:
 			disconnector(h_id)
-		self._update_server.finish()
-		urllib.urlopen("http://localhost:"+str(PlanetView.PORT)+"/") #pings the server, gets it to quit
+		if self._USING_AJAX:
+			import urllib
+			self._update_server.finish()
+			try:
+				urllib.urlopen("http://localhost:"+str(PlanetView.PORT)+"/") #pings the server, gets it to quit
+			except:
+				print 'error closing planetview server'
 		self._render("<html><body></body></html")
-		if utils.RUNNING_SUGAR:
-			_sugar.browser_shutdown()
-		else:
+		if not utils.RUNNING_SUGAR:
 			gtkmozembed.pop_startup()
 		
 	#protected functions
@@ -328,9 +355,12 @@ class PlanetView(gobject.GObject):
 			last_entry = len(self._entrylist)
 			
 		media_exists = False
-		entries = ""
+		entries = []
 		html = ""
 		unreads = []
+		
+		#preload the block of entries, which is nicer to the db
+		self._load_entry_block(self._entrylist[self._first_entry:last_entry])
 
 		i=self._first_entry-1
 		for entry_id in self._entrylist[self._first_entry:last_entry]:
@@ -338,7 +368,7 @@ class PlanetView(gobject.GObject):
 			entry_html, item = self._load_entry(entry_id)
 			if item.has_key('media'):
 				media_exists = True
-			if not item.has_key('media'):
+			else:
 				if self._readinfo:
 					if self._readinfo[i]==0:
 						unreads.append(entry_id)
@@ -359,7 +389,7 @@ class PlanetView(gobject.GObject):
 				p.feed(entry_html)
 				entry_html = p.new_data
 			
-			entries += entry_html
+			entries.append(entry_html)
 
 		self._app.mark_entrylist_as_viewed(unreads, False)
 		for e in unreads:
@@ -369,43 +399,46 @@ class PlanetView(gobject.GObject):
 				print "warning: can't remove non-existant entry from store"
 			
 		#######build HTML#######	
-		html = self._build_header(media_exists)
+		html = []
+		html.append(self._build_header(media_exists))
 		
-		html += self._custom_message
+		html.append(self._custom_message)
 		
-		html += """<div id="nav_bar"><table
+		html.append("""<div id="nav_bar"><table
 					style="width: 100%; text-align: left; margin-left: auto; margin-right: auto;"
  					border="0" cellpadding="2" cellspacing="0">
 					<tbody>
-					<tr><td>"""
+					<tr><td>""")
 		if self._first_entry > 0:
-			html += '<a href="planet:up">Newer Entries</a>'
-		html += '</td><td style="text-align: right;">'
+			html.append('<a href="planet:up">Newer Entries</a>')
+		html.append('</td><td style="text-align: right;">')
 		if last_entry < len(self._entrylist):
-			html += '<a href="planet:down">Older Entries</a>'
-		html += "</td></tr></tbody></table></div>"
+			html.append('<a href="planet:down">Older Entries</a>')
+		html.append("</td></tr></tbody></table></div>")
 		
 		if self._state != S_SEARCH:
-		#if not self._showing_search: 
-			html += '<div class="feed_title">'+self._feed_title+"</div>"
+			html.append('<div class="feed_title">'+self._feed_title+"</div>")
 		html += entries
 			
-		html += """<div id="nav_bar"><table
+		html.append("""<div id="nav_bar"><table
 					style="width: 100%; text-align: left; margin-left: auto; margin-right: auto;"
 					border="0" cellpadding="2" cellspacing="0">
 					<tbody>
-					<tr><td>"""
+					<tr><td>""")
 		if self._first_entry > 0:
-			html += '<a href="planet:up">Newer Entries</a>'
-		html += '</td><td style="text-align: right;">'
+			html.append('<a href="planet:up">Newer Entries</a>')
+		html.append('</td><td style="text-align: right;">')
 		if last_entry < len(self._entrylist):
-			html += '<a href="planet:down">Older Entries</a>'
-		html += "</td></tr></tbody></table></div>"
-		html += "</body></html>"
+			html.append('<a href="planet:down">Older Entries</a>')
+		html.append("</td></tr></tbody></table></div>")
+		html.append("</body></html>")
+		
+		html = "".join(html)
+		
 		self._render(html)
 	
 	def _build_header(self, media_exists):
-		if self._renderer == MOZILLA or self._renderer == DEMOCRACY_MOZ:
+		if self._renderer == MOZILLA:
 			html = (
             """<html><head>
             <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
@@ -418,7 +451,7 @@ class PlanetView(gobject.GObject):
 									   self._moz_font, 
 									   self._moz_size, 
 									   self._css)
-			if media_exists:
+			if media_exists and self._USING_AJAX:
 				html += """
 	            <script type="text/javascript">
 	            <!--
@@ -494,41 +527,66 @@ class PlanetView(gobject.GObject):
 			
 		return html
 		
+	def _load_entry_block(self, entry_id_list):
+		for entry in entry_id_list:
+			if self._entry_store.has_key(entry):
+				entry_id_list.remove(entry)
+		
+		if len(entry_id_list) == 0:
+			return
+				
+		entries = self._db.get_entry_block(entry_id_list)
+		media = self._db.get_entry_media_block(entry_id_list)
+		for item in entries:
+			if media.has_key(item['entry_id']):
+				item['media'] = media[item['entry_id']]
+				if self._USING_AJAX:
+					ret = []
+					ret.append(str(item['entry_id'])+" ")
+					for medium in item['media']:
+						ret += htmlify_media(medium, self._mm)
+					ret = "".join(ret)
+					self._update_server.push_update(ret)
+				
+			if self._state == S_SEARCH:
+				item['feed_title'] = self._db.get_feed_title(item['feed_id'])
+				self._entry_store[item['entry_id']] = (htmlify_item(item, mm=self._mm, ajax=self._USING_AJAX, with_feed_titles=True, indicate_new=True, basic_progress=not self._USING_AJAX),item)
+			else:
+				self._entry_store[item['entry_id']] = (htmlify_item(item, mm=self._mm, ajax=self._USING_AJAX, indicate_new=True, basic_progress=not self._USING_AJAX),item)
+				
 	def _load_entry(self, entry_id, force = False):
 		if self._entry_store.has_key(entry_id) and not force:
+			#print "loaded:", self._entry_store[entry_id]
 			return self._entry_store[entry_id]
 		
 		item = self._db.get_entry(entry_id)
 		media = self._db.get_entry_media(entry_id)
-		read = self._db.get_entry_read(entry_id)
 		if media:
 			item['media']=media
-		item['read'] = read
 		if self._state == S_SEARCH:
-		#if self._showing_search:
 			item['feed_title'] = self._db.get_feed_title(item['feed_id'])
-			self._entry_store[entry_id] = (htmlify_item(item, ajax=True, with_feed_titles=True, indicate_new=True),item)
+			self._entry_store[entry_id] = (htmlify_item(item, mm=self._mm, ajax=self._USING_AJAX, with_feed_titles=True, indicate_new=True, basic_progress=not self._USING_AJAX),item)
 		else:
-			self._entry_store[entry_id] = (htmlify_item(item, ajax=True, indicate_new=True),item)
+			self._entry_store[entry_id] = (htmlify_item(item, mm=self._mm, ajax=self._USING_AJAX, indicate_new=True, basic_progress=not self._USING_AJAX),item)
 		
 		index = self._entrylist.index(entry_id)
 		if index >= self._first_entry and index <= self._first_entry+ENTRIES_PER_PAGE:
 			entry = self._entry_store[entry_id][1]
 			if not entry.has_key('media'):
 				return self._entry_store[entry_id]
-			ret = []
-			ret.append(str(entry_id)+" ")
-			for medium in entry['media']:
-				ret += htmlify_media(medium, self._mm)
-			ret = "".join(ret)
-			self._update_server.push_update(ret)
+			if self._USING_AJAX:
+				ret = []
+				ret.append(str(entry_id)+" ")
+				for medium in entry['media']:
+					ret += htmlify_media(medium, self._mm)
+				ret = "".join(ret)
+				self._update_server.push_update(ret)
 		
 		return self._entry_store[entry_id]
 		
 	def _render(self, html):
 		if self._moz_realized:
 			self._moz.open_stream("http://localhost:"+str(PlanetView.PORT),"text/html")
-			#self._moz.open_stream("file:///","text/html")
 			while len(html)>60000:
 					part = html[0:60000]
 					html = html[60000:]
@@ -548,8 +606,13 @@ class PlanetView(gobject.GObject):
 			self.emit('link-activated', link)
 		return True #don't load url please
 		
+	def _moz_new_window(self, mozembed, retval, chromemask):
+		# hack to try to properly load links that want a new window
+		self.emit('link-activated', mozembed.get_link_message())
+		
 	def _moz_realize(self, widget, realized):
 		self._moz_realized = realized
+		self.display_item()
 		
 	def _moz_link_message(self, data):
 		self._main_window.display_status_message(self._moz.get_link_message())
@@ -576,72 +639,6 @@ class PlanetView(gobject.GObject):
 		self._moz_font = "'"+self._moz_font+"','"+" ".join(map(str, [x for x in moz_font.split() if isValid(x)])) + "',Arial"
 		self._moz_size = int([x for x in moz_font.split() if isNumber(x)][-1])+4
 		
-	class MyTCPServer(SocketServer.ForkingTCPServer):
-		def __init__(self, server_address, RequestHandlerClass):
-			SocketServer.ForkingTCPServer.__init__(self, server_address, RequestHandlerClass)
-			
-			self._key = ""
-			self.generate_key()
-			
-			self._updates = []
-			self._quitting = False
-			
-		def serve_forever(self):
-			while 1:
-				self.handle_request()
-				if self._quitting:
-					logging.info('quitting tcp server')
-					return
-				#if len(self._updates)>0:
-					#We must have posted an update.  So pop it (unlike in the request handler,
-					#changes actually have an effect here!)
-					#self._updates.pop(0)
-					#self._updates = []
-					#print "popped all"
-					#logging.info('popped all')
-					
-		def finish(self):
-			self._quitting = True
-					
-		def generate_key(self):
-			self._key = str(random.randint(1,1000000))
-			return self._key
-			
-		def get_key(self):
-			return self._key
-			
-		def push_update(self, update):
-			remove_list = [u for u in self._updates if u.split(" ")[0] == update.split(" ")[0]]
-			for item in remove_list:
-				self._updates.remove(item)
-			self._updates.append(update)
-			
-		def peek_update(self):
-			return self._updates[0]
-			
-		def peek_all(self):
-			return "\n".join(self._updates)
 
-		def clear_updates(self):
-			self._updates = []
-			
-		def update_count(self):
-			return len(self._updates)
 					
-	class EntryInfoServer(SimpleHTTPServer.SimpleHTTPRequestHandler):	
-		"""for some reason, any variable I change in this class changes RIGHT FUCKING BACK
-		as soon as it exits.  So we don't actually pop the value here"""
 
-		def __init__(self, request, client_address, server):
-			SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self, request, client_address, server)
-			
-		def do_GET(self):
-			key = self.path[1:] #strip leading /
-			if key != self.server.get_key():
-				self.wfile.write("PenguinTV Unauthorized")
-				return
-			if self.server.update_count()==0:
-				self.wfile.write("")
-			else:
-				update = self.server.peek_all()
-				self.wfile.write(update)
