@@ -8,20 +8,20 @@
 
 #import code
 #from sizer import scanner
-#objs = scanner.Objects()
-#code.interact(local = {'objs': objs})
-#from sizer import formatting
-
+##objs = scanner.Objects()
+##code.interact(local = {'objs': objs})
+##from sizer import formatting
 
 #import urlparse loaded as needed
 from pysqlite2.dbapi2 import OperationalError as OperationalError
 import threading
 import sys, os, os.path
 import gc
+#gc.set_debug(gc.DEBUG_STATS | gc.DEBUG_SAVEALL)
 import logging
-import traceback
 import urllib
 try:
+	import gnome
 	import gnome.ui
 	HAS_GNOME=True
 except:
@@ -39,16 +39,13 @@ import gtk.glade
 import gobject
 import locale
 import gettext
-	
+
 locale.setlocale(locale.LC_ALL, '')
 gettext.install('penguintv', '/usr/share/locale')
 gettext.bindtextdomain('penguintv', '/usr/share/locale')
 gettext.textdomain('penguintv')
 _=gettext.gettext
 
-#DEBUG (see also utils.py for some debugs)
-_FORCE_DEMOCRACY_MOZ=False
-
 DOWNLOAD_ERROR=0
 DOWNLOAD_PROGRESS=1
 DOWNLOAD_WARNING=2
@@ -66,13 +63,19 @@ import AddFeedDialog
 import PreferencesDialog
 import MainWindow, FeedList, EntryList, EntryView
 
+if utils.HAS_STATUS_ICON:
+	import PtvTrayIcon
+
 CANCEL=0
 PAUSE=1
 
 REFRESH_SPECIFIED=0
 REFRESH_AUTO=1
 
-AUTO_REFRESH_FREQUENCY=5*60*1000
+if utils.RUNNING_SUGAR:
+	AUTO_REFRESH_FREQUENCY=30*60*1000
+else:
+	AUTO_REFRESH_FREQUENCY=5*60*1000
 
 #states:
 DEFAULT            = 1
@@ -86,7 +89,7 @@ class PenguinTVApp(gobject.GObject):
 	__gsignals__ = {
 		'feed-polled': (gobject.SIGNAL_RUN_FIRST, 
                            gobject.TYPE_NONE, 
-                           ([gobject.TYPE_INT])),
+                           ([gobject.TYPE_INT, gobject.TYPE_PYOBJECT])),
         'feed-added': (gobject.SIGNAL_RUN_FIRST, 
                            gobject.TYPE_NONE, 
                            ([gobject.TYPE_INT, gobject.TYPE_BOOLEAN])),
@@ -95,7 +98,23 @@ class PenguinTVApp(gobject.GObject):
                            ([gobject.TYPE_INT])),
 		'entry-updated': (gobject.SIGNAL_RUN_FIRST, 
                            gobject.TYPE_NONE, 
-                           ([gobject.TYPE_INT, gobject.TYPE_INT]))
+                           ([gobject.TYPE_INT, gobject.TYPE_INT])),
+		'notify-tags-changed': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([])),
+		# the integer here is really just so I can avoid a circular codepath
+		# in tag editor ng
+		'tags-changed': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([gobject.TYPE_INT])),                        
+		'download-finished': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([gobject.TYPE_PYOBJECT])),
+		'app-loaded': (gobject.SIGNAL_RUN_FIRST, 
+                           gobject.TYPE_NONE, 
+                           ([])),
+		'setting-changed':(gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+						   ([gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT]))
 	}
 
 	def __init__(self, window=None):
@@ -117,32 +136,22 @@ class PenguinTVApp(gobject.GObject):
 		found_glade = False
 		
 		
-		for p in (os.path.join(utils.GetPrefix(),"share","penguintv"),
-				  os.path.join(utils.GetPrefix(),"share"),
-				  os.path.join(os.path.split(os.path.abspath(sys.argv[0]))[0],"share"),
-				  os.path.join(utils.GetPrefix(),"share","sugar","activities","ptv","share"),
-				  os.path.join(os.path.split(os.path.split(utils.__file__)[0])[0],'share')):
-			try:
-				self.glade_prefix = p
-				os.stat(os.path.join(self.glade_prefix,"penguintv.glade"))
-				found_glade = True
-				break
-			except:
-				continue
-		if not found_glade:
+		self.glade_prefix = utils.get_glade_prefix()
+		if self.glade_prefix is None:
 			print "error finding glade file."
 			sys.exit()
 						
 		logging.info("penguintv "+utils.VERSION+" startup")
 			
-		self.db = ptvDB.ptvDB(self._polling_callback)
+		self.db = ptvDB.ptvDB(self._polling_callback, self._emit_change_setting)
+		
 		self._firstrun = self.db.maybe_initialize_db()
 
 		self.db.clean_media_status()
 		self.mediamanager = MediaManager.MediaManager(self._progress_callback, self._finished_callback)
-		self._updater_db = None
 		self._polled=0      #Used for updating the polling progress bar
-		self._polling_taskid=0 #the taskid we can use to waitfor a polling operation
+		self._polling_taskinfo = -1 # the taskid we can use to waitfor a polling operation,
+									# and the time of last polling
 		self.polling_frequency=12*60*60*1000
 		self._bt_settings = {}
 		self._exiting=0
@@ -165,7 +174,12 @@ class PenguinTVApp(gobject.GObject):
 			use_internal_player = self.db.get_setting(ptvDB.BOOL, '/apps/penguintv/use_internal_player', True)
 		else:
 			use_internal_player = True
-		self.main_window = MainWindow.MainWindow(self,self.glade_prefix, use_internal_player, window=window) 
+			
+		self._status_icon = None
+		if utils.HAS_STATUS_ICON:
+			self._status_icon = PtvTrayIcon.PtvTrayIcon(self, utils.get_icon_filename())	
+			
+		self.main_window = MainWindow.MainWindow(self, self.glade_prefix, use_internal_player, window=window, status_icon=self._status_icon) 
 		self.main_window.layout=window_layout
 		
 		#some signals
@@ -175,7 +189,7 @@ class PenguinTVApp(gobject.GObject):
 		"""After we have Show()n the main window, set up some more stuff"""
 		#gtk.gdk.threads_enter()
 		gst_player = self.main_window.get_gst_player()
-		self._player = Player.Player(gst_player)
+		self.player = Player.Player(gst_player)
 		if gst_player is not None:
 			gst_player.connect('item-not-supported', self._on_item_not_supported)
 		self._gui_updater = UpdateTasksManager.UpdateTasksManager(UpdateTasksManager.GOBJECT, "gui updater")
@@ -194,9 +208,6 @@ class PenguinTVApp(gobject.GObject):
 		self.window_add_feed.hide()
 		self.window_preferences = PreferencesDialog.PreferencesDialog(gtk.glade.XML(self.glade_prefix+'/penguintv.glade', "window_preferences",'penguintv'),self) #MAGIC
 		self.window_preferences.hide()
-		#self.layout_changing_dialog = gtk.glade.XML(self.glade_prefix+'/penguintv.glade', "window_changing_layout",'penguintv').get_widget("window_changing_layout")
-		#self.layout_changing_dialog.connect("delete-event",self.on_window_changing_layout_delete_event)
-		#self.layout_changing_dialog.hide()
 					
 		#gconf
 		if utils.HAS_GCONF:
@@ -211,15 +222,20 @@ class PenguinTVApp(gobject.GObject):
 			conf.notify_add('/apps/penguintv/feed_refresh_frequency',self._gconf_set_polling_frequency)
 			conf.notify_add('/apps/penguintv/app_window_layout',self._gconf_set_app_window_layout)
 			conf.notify_add('/apps/penguintv/feed_refresh_method',self._gconf_set_feed_refresh_method)
+			conf.notify_add('/apps/penguintv/show_notifications',self._gconf_set_show_notifications)
 			conf.notify_add('/apps/penguintv/auto_download',self._gconf_set_auto_download)
+			conf.notify_add('/apps/penguintv/show_notification_always',self._gconf_set_show_notification_always)
 			conf.notify_add('/apps/penguintv/auto_download_limiter',self._gconf_set_auto_download_limiter)
 			conf.notify_add('/apps/penguintv/auto_download_limit',self._gconf_set_auto_download_limit)
+
 		self._load_settings()
 		
 		self.feed_list_view = self.main_window.feed_list_view
 		self._entry_list_view = self.main_window.entry_list_view
 		self._entry_view = self.main_window.entry_view
 		
+		self._entry_view.display_item()
+		
 		self._connect_signals()
 		
 		self.main_window.search_container.set_sensitive(False)
@@ -243,6 +259,7 @@ class PenguinTVApp(gobject.GObject):
 			gobject.timeout_add(30*1000,self.do_poll_multiple, 0)
 			
 		#gtk.gdk.threads_leave()
+		self.emit('app-loaded')
 		return False #for idler	
 		
 	def _import_default_feeds(self):
@@ -324,6 +341,11 @@ class PenguinTVApp(gobject.GObject):
 		self._auto_download = val
 		self.window_preferences.set_auto_download(val)
 		
+		val = self.db.get_setting(ptvDB.BOOL, '/apps/penguintv/show_notification_always', True)
+		if utils.HAS_STATUS_ICON:
+			self._status_icon.set_show_always(val)
+		self.window_preferences.set_show_notification_always(val)
+		
 		val = self.db.get_setting(ptvDB.BOOL, '/apps/penguintv/auto_download_limiter', False)
 		self._auto_download_limiter = val
 		self.window_preferences.set_auto_download_limiter(val)
@@ -370,7 +392,7 @@ class PenguinTVApp(gobject.GObject):
 			self.db.set_setting(ptvDB.STRING, '/apps/penguintv/default_filter',self.feed_list_view.filter_name)
 		else:
 			self.db.set_setting(ptvDB.STRING, '/apps/penguintv/default_filter',"")
-		self.db.set_setting(ptvDB.BOOL, '/apps/penguintv/use_internal_player', self._player.using_internal_player())
+		#self.db.set_setting(ptvDB.BOOL, '/apps/penguintv/use_internal_player', self.player.using_internal_player())
 	
 	def resume_resumable(self):
 		list = self.db.get_resumable_media()
@@ -391,12 +413,17 @@ class PenguinTVApp(gobject.GObject):
 		
 	def do_quit(self):
 		"""save and shut down all our threads"""
+		
+		##good breakpoint for gc analysis
+		#import code
+		#code.interact()
+		
 		logging.info('ptv quitting')
 		self._exiting=1
 		self._entry_view.finish()
 		self.feed_list_view.interrupt()
 		self._update_thread.goAway()
-		#self._db_updater.queue(self._updater_thread_db.finish, None)
+		self._updater_thread_db.finish(False)
 		self.main_window.finish()
 		logging.info('stopping downloads')
 		self.stop_downloads()
@@ -415,6 +442,7 @@ class PenguinTVApp(gobject.GObject):
 		#	time.sleep(1)
 		logging.info('stopping socket')
 		self._socket.close()
+		
 		if not utils.RUNNING_SUGAR:
 			gtk.main_quit()
 		
@@ -438,6 +466,15 @@ class PenguinTVApp(gobject.GObject):
 				if was_setup!=self.polling_frequency and was_setup!=0:
 					return False
 
+		if self._polling_taskinfo != -1:
+			#print "I think we are already polling"
+			#print "poll id set:", self._polling_taskinfo, time.time(), "(", time.time() - self._polling_taskinfo, ")"
+			if time.time() - self._polling_taskinfo > 20*60:
+				#print "poll id reset"
+				#print "but it's been an awful long time.  Polling anyway"
+				self._polling_taskinfo = -1
+			else:
+				return True
 		#gtk.gdk.threads_enter()
 
 		self.main_window.update_progress_bar(0,MainWindow.U_POLL)
@@ -450,12 +487,12 @@ class PenguinTVApp(gobject.GObject):
 									(2000, self.main_window.display_status_message, ""), 
 								    task_id, 
 									False)
-		self._polling_taskid = self._gui_updater.queue(self.update_disk_usage, 
+		self._polling_taskinfo = self._gui_updater.queue(self.update_disk_usage, 
 													   None, 
 													   task_id, 
 													   False) #because this is also waiting
 		if self._auto_download == True:
-			self._polling_taskid = self._gui_updater.queue(self._auto_download_unviewed, 
+			self._polling_taskinfo = self._gui_updater.queue(self._auto_download_unviewed, 
 														   None, 
 														   task_id)
 		#gtk.gdk.threads_leave()
@@ -484,6 +521,7 @@ class PenguinTVApp(gobject.GObject):
 				self.emit('entry-updated', d[2], d[3])
 		else:
 			print "we were unable to free up enough space."
+			#print download_list
 		self.update_disk_usage()
 			
 	def _free_media_space(self, size_needed):
@@ -498,9 +536,9 @@ class PenguinTVApp(gobject.GObject):
 		
 		#adjust actual free space so we never fill up the drive
 		if utils.RUNNING_SUGAR:
-			free_buffer = disk_total * .3
+			free_buffer = 300000000 # 300 meg
 		else:
-			free_buffer = disk_total * .05
+			free_buffer =  10000000 # 10 meg
 			
 		size_to_free = 0
 		if self._auto_download_limiter:
@@ -521,6 +559,11 @@ class PenguinTVApp(gobject.GObject):
 		if size_to_free == 0:
 			return True
 			
+		#get the media that's currently in the player so we don't delete it
+		if utils.HAS_GSTREAMER:
+			media_in_player = self.player.get_queue()
+			media_in_player = [m[3] for m in media_in_player]
+			
 		media_to_remove = []
 		removed_size = 0
 		for media_id,entry_id,feed_id,filename,date in self.db.get_deletable_media():
@@ -534,6 +577,12 @@ class PenguinTVApp(gobject.GObject):
 					print "ERROR: didn't free up the space like we thought2",utils.get_disk_free(self.mediamanager.media_dir)
 					return False
 				return True
+				
+			#don't remove anything that's queued in the player
+			if utils.HAS_GSTREAMER:
+				if media_id in media_in_player:
+					continue
+			
 			size = os.stat(filename)[6]
 			removed_size += size
 			print "removing:",filename, size,"bytes for a total of",removed_size
@@ -541,12 +590,12 @@ class PenguinTVApp(gobject.GObject):
 			self.db.set_entry_read(entry_id, True)
 			self.emit('entry-updated', entry_id, feed_id)
 		return False
-			
+		
 	def add_search_tag(self, query, tag_name):
 		self.db.add_search_tag(query, tag_name)
 		#could raise ptvDB.TagAlreadyExists, let it go
 		self._saved_search = self.main_window.search_entry.get_text()
-		self.main_window.update_filters()
+		self.emit('tags-changed', 0)
 		while gtk.events_pending(): #wait for the list to update
 			gtk.main_iteration()
 		index = self.main_window.get_filter_index(tag_name)
@@ -558,7 +607,7 @@ class PenguinTVApp(gobject.GObject):
 			
 	def remove_search_tag(self, tag_name):
 		self.db.remove_tag(tag_name)
-		self.main_window.update_filters()
+		self.emit('tags-changed', 0)
 		while gtk.events_pending():
 			gtk.main_iteration()
 			
@@ -587,8 +636,15 @@ class PenguinTVApp(gobject.GObject):
 			self.db.add_tag_for_feed(feed_id, tag)	
 		if removed_tags or added_tags:
 			self.feed_list_view.set_selected(feed_id)
-		self.main_window.update_filters()
+		self.emit('tags-changed', 0)
 		self.feed_list_view.filter_all(False)
+		if old_tags is not None:
+			if ptvDB.NOTIFYUPDATES in old_tags:
+				self.emit('notify-tags-changed')
+				return #don't need the next test
+		if new_tags is not None:
+			if ptvDB.NOTIFYUPDATES in new_tags:
+				self.emit('notify-tags-changed')
 		
 	def _populate_feeds(self, callback=None, subset=FeedList.ALL):
 		self.set_state(LOADING_FEEDS)
@@ -658,7 +714,7 @@ class PenguinTVApp(gobject.GObject):
 			self.db.set_entry_read(media['entry_id'],True)
 			self.db.set_media_viewed(item,True)
 			if utils.is_known_media(media['file']):
-				self._player.play(media['file'], feed_title + " &#8211; " + entry['title'])
+				self.player.play(media['file'], feed_title + " &#8211; " + entry['title'], media['media_id'])
 			else:
 				if HAS_GNOME:
 					gnome.url_show(media['file'])
@@ -718,9 +774,9 @@ class PenguinTVApp(gobject.GObject):
 			if HAS_GNOME:
 				gnome.url_show(parsed_url[0]+"://"+quoted_url+http_arguments+anchor)
 			elif utils.RUNNING_SUGAR:
-				from sugar.activity import ActivityFactory
-				activity = ActivityFactory.create("org.laptop.WebActivity")
-				activity.execute("load-uri", [parsed_url[0]+"://"+quoted_url+http_arguments+anchor])
+				from sugar.activity import activityfactory
+				uri=parsed_url[0]+"://"+quoted_url+http_arguments+anchor
+				activityfactory.create_with_uri('org.laptop.WebActivity', uri)
 		elif action=="file":
 			print parsed_url[0]+"://"+urllib.quote(parsed_url[1]+parsed_url[2])
 			if HAS_GNOME:
@@ -882,7 +938,7 @@ class PenguinTVApp(gobject.GObject):
 			else:
 				for feed in newfeeds:
 					self.feed_list_view.add_feed(feed)
-			self.main_window.update_filters()
+			self.emit('tags-changed', 0)
 			saved_auto = False
 			self.main_window.display_status_message("")
 			#shut down auto-downloading for now (need to wait until feeds are marked)
@@ -890,7 +946,7 @@ class PenguinTVApp(gobject.GObject):
 				saved_auto = True
 				self._auto_download = False
 			self.do_poll_multiple(feeds=newfeeds)
-			task_id = self._gui_updater.queue(self.__first_poll_marking_list, (newfeeds,saved_auto), self._polling_taskid)
+			task_id = self._gui_updater.queue(self.__first_poll_marking_list, (newfeeds,saved_auto), self._polling_taskinfo)
 			dialog.hide()
 			del dialog
 			if len(newfeeds)==1:
@@ -929,11 +985,11 @@ class PenguinTVApp(gobject.GObject):
 	def _reset_auto_download(self):
 		self._auto_download = True
 			
-	def mark_entry_as_viewed(self,entry, update_entrylist=True):
+	def mark_entry_as_viewed(self,entry, feed_id, update_entrylist=True):
 		self.db.set_entry_read(entry,True)
 		if update_entrylist: #hack for PlanetView
 			self.update_entry_list(entry)
-		self.feed_list_view.mark_entries_read(1)
+		self.feed_list_view.mark_entries_read(1, feed_id)
 		
 	def mark_entrylist_as_viewed(self, entrylist, update_entrylist=True):
 		if len(entrylist) == 0:
@@ -969,21 +1025,25 @@ class PenguinTVApp(gobject.GObject):
 		filelist=[]
 		if media:
 			for medium in media:
-				filelist.append([medium['file'], feed_title + " &#8211; " + entry['title']])
+				filelist.append([medium['file'], feed_title + " &#8211; " + entry['title'], medium['media_id']])
 				self.db.set_media_viewed(medium['media_id'],True)
-		self._player.play_list(filelist)
+		self.player.play_list(filelist)
 		self.emit('entry-updated', entry_id, entry['feed_id'])
 		
 	def play_unviewed(self):
+		#objs = scanner.Objects()
+		#code.interact(local = {'objs': objs})
+		#code.interact()
+		#return
 		playlist = self.db.get_unplayed_media(True) #set viewed
 		playlist.reverse()
-		self._player.play_list([[item[3],item[5] + " &#8211; " + item[4]] for item in playlist])
+		self.player.play_list([[item[3],item[5] + " &#8211; " + item[4], item[0]] for item in playlist])
 		for row in playlist:
 			self.feed_list_view.update_feed_list(row[2],['readinfo'])
 			
-	def _on_item_not_supported(self, player, filename, name):
+	def _on_item_not_supported(self, player, filename, name, userdata):
 		if not utils.RUNNING_SUGAR:
-			self._player.play(filename, name, force_external=True) #retry, force external player
+			self.player.play(filename, name, userdata, force_external=True) #retry, force external player
 		else:
 			dialog = gtk.Dialog(title=_("Unknown file type"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
 			label = gtk.Label("Gstreamer did not recognize this file. (email owen-olpc@ywwg.com for more info)")
@@ -994,12 +1054,11 @@ class PenguinTVApp(gobject.GObject):
 			del dialog
 			return
 
-	def refresh_feed(self,feed):
-		#if event.state & gtk.gdk.SHIFT_MASK:
-		#	print "shift-- shift delete it"
+	def refresh_feed(self, feed):
+		def _refresh_cb(update_data, success):
+			self._threaded_emit('feed-polled', feed, update_data)
 		self.main_window.display_status_message(_("Polling Feed..."))
-		task_id = self._db_updater.queue(self._updater_thread_db.poll_feed,(feed,ptvDB.A_IGNORE_ETAG+ptvDB.A_DO_REINDEX))
-		self._gui_updater.queue(self.emit, ('feed-polled', feed), task_id, False)
+		task_id = self._db_updater.queue(self._updater_thread_db.poll_feed_trap_errors,(feed,  _refresh_cb))
 		
 	def _unset_state(self, authorize=False):
 		"""gets app ready to display new state by unloading current state.
@@ -1065,7 +1124,7 @@ class PenguinTVApp(gobject.GObject):
 			query = query.replace("!","")
 			result = self.db.search(query, blacklist=blacklist)
 		except Exception, e:
-			print "Error with that search term: ", e
+			print "Error with that search term: ", query, e
 			result=([],[])
 		return result
 	
@@ -1161,7 +1220,7 @@ class PenguinTVApp(gobject.GObject):
 
 	def change_filter(self, current_filter, tag_type):
 		filter_id = self.main_window.get_active_filter()[1]
-		if filter_id == FeedList.SEARCH:
+		if utils.HAS_LUCENE and filter_id == FeedList.SEARCH:
 			self._show_search(self._saved_search, self._search(self._saved_search))
 			if self._threaded_searcher:
 				if not self._waiting_for_search:
@@ -1257,7 +1316,7 @@ class PenguinTVApp(gobject.GObject):
 			
 	def get_feed_refresh_method(self):
 		return self.feed_refresh_method
-			
+		
 	def _gconf_set_feed_refresh_method(self, client, *args, **kwargs):
 		refresh = self.db.get_setting(ptvDB.STRING, '/apps/penguintv/feed_refresh_method', 'auto')
 		self.set_feed_refresh_method(refresh, client)
@@ -1291,7 +1350,7 @@ class PenguinTVApp(gobject.GObject):
 		return feed_id
 		
 	def _db_add_feed_cb(self, feed, success):
-		self._threaded_emit('feed-polled', feed['feed_id'])
+		self._threaded_emit('feed-polled', feed['feed_id'], feed)
 		self._threaded_emit('feed-added', feed['feed_id'], success)
 		
 	def __feed_added_cb(self, app, feed_id, success):
@@ -1306,6 +1365,7 @@ class PenguinTVApp(gobject.GObject):
 		this_feed_list = [item for item in all_feeds_list if item[3] == feed_id]
 		for item in this_feed_list[1:]:
 			self.db.set_entry_read(item[2],1)
+			self.emit('entry-updated', item[2], feed_id)
 	
 	def add_feed_filter(self, pointed_feed_id, filter_name, query):
 		try:
@@ -1334,7 +1394,7 @@ class PenguinTVApp(gobject.GObject):
 	def delete_media(self, media_id):
 		"""Deletes specific media id"""
 		self.db.delete_media(media_id)
-		self.main_window.download_finished()
+		self.main_window.update_downloads()
 		self.mediamanager.generate_playlist()
 		self.db.set_media_viewed(media_id,True)
 		self.update_disk_usage()
@@ -1371,8 +1431,13 @@ class PenguinTVApp(gobject.GObject):
 		"""cancels a download and cleans up.  Right now there's redundancy because we call this twice
 		   for files that are downloading -- once when we ask it to stop downloading, and again when the
 		   callback tells the thread to stop working.  how to make this better?"""
-		   
-		self.mediamanager.stop_download(item['media_id'])
+		
+		d = None
+		try:   
+			d = self.mediamanager.get_downloader(item['media_id'])
+			self.mediamanager.stop_download(item['media_id'])
+		except:
+			pass
 		self.db.set_media_download_status(item['media_id'],ptvDB.D_NOT_DOWNLOADED)
 		self.delete_media(item['media_id']) #marks as viewed
 		self.main_window.update_download_progress()
@@ -1390,7 +1455,8 @@ class PenguinTVApp(gobject.GObject):
 			self._populate_feeds(self._done_populating, FeedList.DOWNLOADED)
 			self.feed_list_view.resize_columns()
 		self.feed_list_view.filter_all() #to remove active downloads from the list
-		self.main_window.download_finished()
+		if d is not None:
+			self.emit('download-finished', d)
 		
 	def do_pause_download(self, media_id):
 		self.mediamanager.get_downloader(media_id).pause()
@@ -1408,6 +1474,7 @@ class PenguinTVApp(gobject.GObject):
 		
 	def _download_finished(self, d):
 		"""Process the data from a callback for a downloaded file"""
+		
 		self.update_disk_usage()
 		if d.status==Downloader.FAILURE: 
 			self.db.set_media_download_status(d.media['media_id'],ptvDB.D_ERROR) 
@@ -1415,11 +1482,10 @@ class PenguinTVApp(gobject.GObject):
 			self.main_window.update_download_progress()
 		elif d.status==Downloader.FINISHED or d.status==Downloader.FINISHED_AND_PLAY:
 			if os.stat(d.media['file'])[6] < int(d.media['size']/2) and os.path.isfile(d.media['file']): #don't check dirs
-				dic = {'reported_size': str(d.media['size']),
-					 'actual_size': str(os.stat(d.media['file'])[6])}
 				self.db.set_entry_read(d.media['entry_id'],False)
 				self.db.set_media_viewed(d.media['media_id'],False)
 				self.db.set_media_download_status(d.media['media_id'],ptvDB.D_DOWNLOADED)
+				d.status = Downloader.FAILURE
 			else:
 				self.main_window.update_download_progress()
 				if d.status==Downloader.FINISHED_AND_PLAY:
@@ -1427,12 +1493,12 @@ class PenguinTVApp(gobject.GObject):
 					self.db.set_media_viewed(d.media['media_id'], True)
 					entry = self.db.get_entry(d.media['entry_id'])
 					feed_title = self.db.get_feed_title(entry['feed_id'])
-					self._player.play(d.media['file'], feed_title + " &#8211; " + entry['title'])
+					self.player.play(d.media['file'], feed_title + " &#8211; " + entry['title'], d.media['media_id'])
 				else:
 					self.db.set_entry_read(d.media['entry_id'],False)
 					self.db.set_media_viewed(d.media['media_id'],False)
 				self.db.set_media_download_status(d.media['media_id'],ptvDB.D_DOWNLOADED)	
-			self.main_window.download_finished() #FIXME: convert to gobject signal one day
+		self.emit('download-finished', d)
 		if self._exiting:
 			self.feed_list_view.filter_all() #to remove active downloads from the list
 			return
@@ -1482,6 +1548,22 @@ class PenguinTVApp(gobject.GObject):
 	def set_auto_download(self, auto_download):
 		self._auto_download = auto_download
 		
+	def _gconf_set_show_notification_always(self, client, *args, **kwargs):
+		show_notification_always = client.get_bool('/apps/penguintv/show_notification_always')
+		self.window_preferences.set_show_notification_always(show_notification_always)
+		if utils.HAS_STATUS_ICON:
+			self._status_icon.set_show_always(show_notification_always)
+		
+	def set_show_notification_always(self, show_notification_always):
+		if utils.HAS_STATUS_ICON:
+			self._status_icon.set_show_always(show_notification_always)
+			
+	def _gconf_set_show_notifications(self, client, *args, **kwargs):
+		show_notifications = client.get_bool('/apps/penguintv/show_notifications')
+		self.emit('setting-changed', ptvDB.BOOL, 
+		          '/apps/penguintv/show_notifications',
+		          show_notifications)
+		
 	def _gconf_set_auto_download_limiter(self, client, *args, **kwargs):
 		auto_download_limiter = client.get_bool('/apps/penguintv/auto_download_limiter')
 		self.set_auto_download_limiter(auto_download_limiter)
@@ -1540,7 +1622,7 @@ class PenguinTVApp(gobject.GObject):
 	def _progress_callback(self,d):
 		"""Callback for downloads.  Not in main thread, so shouldn't generate gtk calls"""
 		if self._exiting == 1:
-			self._gui_updater.queue(self.do_cancel_download,d.media['media_id'], None, True, 1)
+			self._gui_updater.queue(self.do_cancel_download,d.media, None, True, 1)
 			return 1 #returning one is what interrupts the download
 		
 		if d.media.has_key('size_adjustment'):
@@ -1555,34 +1637,39 @@ class PenguinTVApp(gobject.GObject):
 		
 	def _polling_callback(self, args):
 		if not self._exiting:
-			feed_id,update_data,total = args
-			self._gui_updater.queue(self._poll_update_progress,total)
+			feed_id, update_data, total = args
 			if len(update_data)>0:
 				if update_data.has_key('ioerror'):
+					print "ioerror polling reset"
 					self._updater_thread_db.interrupt_poll_multiple()
-					self._gui_updater.queue(self._poll_update_progress, (total, True, _("Trouble connecting to internet")))
-				elif update_data['pollfail']==False:
-					self._gui_updater.queue(self.feed_list_view.update_feed_list, (feed_id,['readinfo','icon','image'],update_data))
-					
-					if not self._showing_search:
-						self._gui_updater.queue(self._entry_list_view.populate_if_selected, feed_id)
+					self._polled = 0
+					self._polling_taskinfo = -1
+					self.main_window.update_progress_bar(-1, MainWindow.U_POLL)
+					self.main_window.display_status_message(_("Trouble connecting to the internet"),MainWindow.U_POLL)
+					gobject.timeout_add(2000, self.main_window.display_status_message,"")
+					return
 				else:
-					self._gui_updater.queue(self.feed_list_view.update_feed_list, (feed_id,['icon'],update_data))
+					update_data['polling_multiple'] = True
+					self._threaded_emit('feed-polled', feed_id, update_data)
 			else:
 				#check image just in case
 				self._gui_updater.queue(self.feed_list_view.update_feed_list, (feed_id,['image']))
+			self._gui_updater.queue(self._poll_update_progress,total)
 		
 	def _poll_update_progress(self, total=0, error = False, errmsg = ""):
 		"""Updates progress for do_poll_multiple, and also displays the "done" message"""
+
 		if error:
 			self._polled=0
+			self._polling_taskinfo = -1
 			self.main_window.update_progress_bar(-1,MainWindow.U_POLL)
 			self.main_window.display_status_message(errmsg,MainWindow.U_POLL)
 			gobject.timeout_add(2000, self.main_window.display_status_message,"")
 			return
 		self._polled += 1
 		if self._polled == total:
-			self._polled=0
+			self._polled = 0
+			self._polling_taskinfo = -1
 			self.main_window.update_progress_bar(-1,MainWindow.U_POLL)
 			self.main_window.display_status_message(_("Feeds Updated"),MainWindow.U_POLL)
 			gobject.timeout_add(2000, self.main_window.display_status_message,"")
@@ -1611,6 +1698,9 @@ class PenguinTVApp(gobject.GObject):
 	def _reset_db_updater(self, db):
 		self._updater_thread_db = db
 		
+	def _emit_change_setting(self, typ, datum, value):
+		self.emit('setting-changed', typ, datum, value)
+		
 	def _threaded_emit(self, signal, *args):
 		def do_emit(signal, *args):
 			gtk.gdk.threads_enter()
@@ -1691,12 +1781,12 @@ def do_quit(self, event, app):
         
 if __name__ == '__main__': # Here starts the dynamic part of the program 
 	if HAS_GNOME:
+		gtk.gdk.threads_init()
 		gnome.init("PenguinTV", utils.VERSION)
 		app = PenguinTVApp()    # Instancing of the GUI
 		if not app._socket.is_server:
 			sys.exit(0)
 		app.main_window.Show() 
-		gtk.gdk.threads_init()
 		#import profile
 		#profile.run('gtk.main()', '/tmp/penguintv-prof')
 		if utils.is_kde():
