@@ -4,12 +4,12 @@
 from pysqlite2 import dbapi2 as sqlite
 from math import floor,ceil
 from random import randint
-import pysqlite2
-#import feedparser  loaded when needed
+import feedparser
 import time
 import string
 import sha
-import urllib, urllib2#, urlparse loaded as needed
+import urllib, urlparse
+from urllib2 import URLError
 from types import *
 import threading
 import ThreadPool
@@ -39,6 +39,11 @@ if utils.HAS_GCONF:
 	import gobject
 if utils.HAS_PYXML:
 	import OPML
+if utils.RUNNING_SUGAR:
+	USING_FLAG_CACHE = False
+else:
+	USING_FLAG_CACHE = True
+#USING_FLAG_CACHE = False
 	
 NEW = 0
 EXISTS = 1
@@ -49,7 +54,10 @@ BOOL    = 1
 INT     = 2
 STRING  = 3
 
-MAX_ARTICLES = 1000
+if utils.RUNNING_SUGAR:
+	MAX_ARTICLES = 50
+else:
+	MAX_ARTICLES = 1000
 
 _common_unicode = { u'\u0093':u'"', u'\u0091': u"'", u'\u0092': u"'", u'\u0094':u'"', u'\u0085':u'...'}
 
@@ -87,6 +95,7 @@ T_BUILTIN = 3
 NOAUTODOWNLOAD="noautodownload"
 NOSEARCH="nosearch"
 NOAUTOEXPIRE="noautoexpire"
+NOTIFYUPDATES="notify"
 
 DB_FILE="penguintv3.db"
 
@@ -96,7 +105,7 @@ from formatter import NullFormatter
 class ptvDB:
 	entry_flag_cache = {}
 	
-	def __init__(self, polling_callback=None):#,username,password):	
+	def __init__(self, polling_callback=None, change_setting_cb=None):
 		if utils.RUNNING_SUGAR:
 			import sugar.env
 			self.home = os.path.join(sugar.env.get_profile_path(), 'penguintv')
@@ -109,9 +118,11 @@ class ptvDB:
 				os.mkdir(self.home)
 			except:
 				raise DBError, "error creating directories: "+self.home
+		new_db = False
 		try:	
-			#also change db connection in pool poll
+			#also check db connection in _process_feed
 			if os.path.isfile(os.path.join(self.home,"penguintv3.db")) == False:
+				new_db = True
 				if os.path.isfile(os.path.join(self.home,"penguintv2.db")):
 					try: 
 						shutil.copyfile(os.path.join(self.home,"penguintv2.db"), os.path.join(self.home,"penguintv3.db"))
@@ -122,8 +133,7 @@ class ptvDB:
 						shutil.copyfile(os.path.join(self.home,"penguintv.db"), os.path.join(self.home,"penguintv3.db"))
 					except:
 						raise DBError,"couldn't create new database file"
-			self._db=sqlite.connect(os.path.join(self.home,"penguintv3.db"), timeout=10	)
-			self._db.isolation_level="DEFERRED"
+			self._db=sqlite.connect(os.path.join(self.home,"penguintv3.db"), timeout=30.0, isolation_level="IMMEDIATE")
 		except:
 			raise DBError,"error connecting to database"
 		
@@ -131,21 +141,21 @@ class ptvDB:
 		self._c.execute('PRAGMA synchronous="NORMAL"')
 		self.cache_dirty = True
 		try:
-			#self._db_execute(self._c, u'SELECT value FROM settings WHERE data="feed_cache_dirty"')
-			#if self._c.fetchone()[0] == 0:
-			#	self.cache_dirty = False
-			self.cache_dirty = self.get_setting(BOOL, "feed_cache_dirty", True)
+			if not new_db:
+				self.cache_dirty = self.get_setting(BOOL, "feed_cache_dirty", True)
 		except:
 			pass
 			
 		self._exiting = False
 		self._cancel_poll_multiple = False
 			
-		if polling_callback==None:
+		if polling_callback is None:
 			self.polling_callback=self._polling_callback
 		else:
 			self.polling_callback = polling_callback		
 			
+		self._change_setting_cb = change_setting_cb
+			
 		if utils.HAS_LUCENE:
 			self.searcher = Lucene.Lucene()
 			
@@ -154,30 +164,32 @@ class ptvDB:
 			
 		self._icon_manager = IconManager.IconManager(self.home)
 			
+		self._blacklist = []
 		try:
-			self._blacklist = self.get_feeds_for_tag(NOSEARCH)
+			if not new_db:
+				self._blacklist = self.get_feeds_for_tag(NOSEARCH)
 		except:
-			self._blacklist = []
+			pass
 		self._reindex_entry_list = []
 		self._reindex_feed_list = []
 		self._filtered_entries = {}
 		self._parse_list = []
 		
 	def _db_execute(self, c, command, args=()):
-		#if "MEDIA" in command.upper() and "SELECT" not in command.upper(): 
+		#if "UPDATE entries" in command.upper(): 
 		#traceback.print_stack()
 		#print command, args
 		try:
 			return c.execute(command, args)
 		except Exception, e:
-			print "barf:",
+			print "Database error:",
 			print command, args
 			raise e
 				
 	def __del__(self):
 		self.finish()
 		
-	def finish(self):
+	def finish(self, closeok=True):
 		self._exiting=True
 		if utils.HAS_LUCENE:
 			if len(self._reindex_entry_list) > 0 or len(self._reindex_feed_list) > 0:
@@ -186,7 +198,10 @@ class ptvDB:
 				self.reindex(threaded=False) #it's usually not much...
 			self.searcher.finish(False)
 		#FIXME: lame, but I'm being lazy
-		if randint(1,10) == 1:
+		#if randint(1,100) == 1:
+		#	print "cleaning up unreferenced media"
+		#	self.clean_file_media()
+		if randint(1,10) == 1 and closeok:
 			print "compacting database"
 			self._c.execute('VACUUM')
 			self._c.close()
@@ -196,7 +211,7 @@ class ptvDB:
 		try:
 			self._db_execute(self._c, u'SELECT * FROM feeds')
 		except:
-			print "create a database"
+			print "initializing database"
 			self._init_database()
 			return True	
 			
@@ -263,14 +278,14 @@ class ptvDB:
 		try:
 			self._db_execute(self._c, u'ALTER TABLE entries ADD COLUMN fakedate DATE')
 			self._db_execute(self._c, u'UPDATE entries SET fakedate = date')
-		except pysqlite2.dbapi2.OperationalError,e:
+		except sqlite.OperationalError,e:
 			if e != "duplicate column name: fakedate":
 				print e #else pass
 			#change db_ver (last thing)
 		self._db_execute(self._c, u'ALTER TABLE feeds ADD COLUMN pollfreq INT')
 		self._db_execute(self._c, u'UPDATE feeds SET pollfreq=1800')
 		self._db_execute(self._c, u'ALTER TABLE feeds ADD COLUMN lastpoll DATE')
-		self._db_execute(self._c, u'UPDATE feeds SET lastpoll=?',(time.time()-(30*60),))
+		self._db_execute(self._c, u'UPDATE feeds SET lastpoll=?',(int(time.time())-(30*60),))
 		self._db_execute(self._c, u'ALTER TABLE feeds ADD COLUMN newatlast INT')
 		self._db_execute(self._c, u'UPDATE feeds SET newatlast=0')
    
@@ -334,31 +349,6 @@ class ptvDB:
 		print "done"
 		
 	def _init_database(self):
-		try:
-			self._db_execute(self._c, u'DROP TABLE settings')
-		except:
-			pass	
-			
-		try:
-			self._db_execute(self._c, u'DROP TABLE feeds')
-		except:
-			pass
-			
-		try:
-			self._db_execute(self._c, u'DROP TABLE entries')
-		except:
-			pass
-			
-		try:
-			self._db_execute(self._c, u'DROP TABLE media')
-		except:
-			pass
-			
-		try:
-			self._db_execute(self._c, u'DROP TABLE fulltext')
-		except:
-			pass
-			
 		self._db_execute(self._c, u"""CREATE TABLE settings
 							(
 								id INTEGER PRIMARY KEY,
@@ -438,6 +428,9 @@ class ptvDB:
 							term,
 							frequency INT);""")
 							
+		self._db_execute(self._c, u"""CREATE INDEX pollindex ON entries (date DESC);""")
+		self._db_execute(self._c, u"""CREATE INDEX feedindex ON feeds (title DESC);""")
+							
 		self._db.commit()
 		
 		self._db_execute(self._c, u"""INSERT INTO settings (data, value) VALUES ("db_ver",4)""")
@@ -513,6 +506,8 @@ class ptvDB:
 			else:
 				self._db_execute(self._c, u'UPDATE settings SET value=? WHERE data=?', (value,datum))
 			self._db.commit()
+		if self._change_setting_cb is not None:
+			self._change_setting_cb(type, datum, value)
 			
 	def set_feed_cache(self, cachelist):
 		"""Cachelist format:
@@ -556,7 +551,7 @@ class ptvDB:
 			d={ 'title':_("Waiting for first poll"),
 				'description':_("This feed has not yet been polled successfully.  There might be an error with this feed.<br>"+str(url)),
 			  }
-			self._db_execute(self._c, u'INSERT INTO entries (id, feed_id, title, creator, description, read, fakedate, date, guid, link, old, new) VALUES (NULL, ?, ?, NULL, ?, ?, 0, ?, ?, "http://", "0", "0")',(feed_id,d['title'],d['description'],'0',time.time(),time.time()))
+			self._db_execute(self._c, u'INSERT INTO entries (id, feed_id, title, creator, description, read, fakedate, date, guid, link, old, new) VALUES (NULL, ?, ?, NULL, ?, ?, 0, ?, ?, "http://", "0", "0")',(feed_id,d['title'],d['description'],'0', int(time.time()), int(time.time())))
 			self._db.commit()
 		else:
 			self._db_execute(self._c, """SELECT id FROM feeds WHERE url=?""",(url,))
@@ -664,7 +659,7 @@ class ptvDB:
 	def poll_multiple(self, arguments=0, feeds=None):
 		"""Polls multiple feeds multithreadedly"""
 		successes=[]
-		cur_time = time.time()
+		cur_time = int(time.time())
 		
 		if feeds is None:
 			if arguments & A_AUTOTUNE and arguments & A_ALL_FEEDS == 0:
@@ -698,28 +693,36 @@ class ptvDB:
 		#grow the cache while we do this operation
 		self._db_execute(self._c, 'PRAGMA cache_size=6000')
 		while polled < len(feeds):
-			if self._exiting:
+			if self._cancel_poll_multiple or self._exiting:
 				break
 			if len(self._parse_list) > 0:
 				polled+=1
 				feed_id, args, total, parsed = self._parse_list.pop(0)
 				self.polling_callback(self._process_feed(feed_id, args, total, parsed))
+				gc.collect()
 			time.sleep(.1)
 		self._db_execute(self._c, 'PRAGMA cache_size=2000')
 		
-			
-		while pool.getTaskCount()>0: #manual joinAll so we can check for exit
-			if self._exiting:
-				pool.joinAll(False,True)
-				self._c.close()
-				self._db.close()
-				return
-			time.sleep(.5)
+		#print "done polling, join now"
+		
+		if self._cancel_poll_multiple:
+			self._parse_list = []
+		else: # no need for manual join
+			while pool.getTaskCount()>0: #manual joinAll so we can check for exit
+				if self._exiting:
+					pool.joinAll(False, True)
+					del pool
+					self._c.close()
+					self._db.close()
+					return
+				time.sleep(.5)
 		pool.joinAll(False,True) #just to make sure I guess
+		#print "joined"
 		del pool
 		self.reindex()
 		self._cancel_poll_multiple = False
 		gc.collect()
+		#print "and out"
 		
 	def interrupt_poll_multiple(self):
 		self._cancel_poll_multiple = True
@@ -732,8 +735,19 @@ class ptvDB:
 		feed_id, arguments, total, data = args
 		url,modified,etag=data
 		
+		#save ram by not piling up polled data
+		if utils.RUNNING_SUGAR:
+			parse_list_limit = 2
+		else:
+			parse_list_limit = 5
+		
+		while len(self._parse_list) > parse_list_limit and not self._exiting:
+			time.sleep(1)
+			
+		if self._exiting:
+			return (feed_id, arguments, total, -1)
+		
 		try:
-			import feedparser
 			#feedparser.disableWellFormedCheck=1  #do we still need this?  it used to cause crashes
 			if arguments & A_IGNORE_ETAG == A_IGNORE_ETAG:
 				data = feedparser.parse(url)
@@ -753,27 +767,29 @@ class ptvDB:
 		try:
 			#poll_arguments = args[1]
 			if self._exiting:
-				return (feed_id,{'pollfail':True}, total)
+				return (feed_id,{'ioerror':None, 'pollfail':False}, total)
 				
 			result = self.poll_feed(feed_id, args, preparsed=data)
 
 			if self._exiting:
-				return (feed_id,{'pollfail':True} ,total)
+				return (feed_id,{'ioerror':None, 'pollfail':False}, total)
 		except sqlite.OperationalError, e:
 			print "Database warning...", e
 			if recurse < 2:
 				time.sleep(5)
 				print "trying again..."
 				self._db.close()
-				self._db=sqlite.connect(os.path.join(self.home,"penguintv3.db"), timeout=10)
+				self._db=sqlite.connect(os.path.join(self.home,"penguintv3.db"), timeout=30, isolation_level="IMMEDIATE")
 				self._c = self._db.cursor()
 				return self._process_feed(feed_id, args, total, data, recurse+1) #and reconnect
 			print "can't get lock, giving up"
 			return (feed_id,{'pollfail':True}, total)
 		except FeedPollError,e:
+			#print "feed poll error",
 			print e
 			return (feed_id,{'pollfail':True}, total)
 		except IOError, e:
+			#print "io error",
 			print e
 			#we got an ioerror, but we won't take it out on the feed
 			return (feed_id,{'ioerror':e, 'pollfail':False}, total)
@@ -789,7 +805,8 @@ class ptvDB:
 		#assemble our handy dictionary while we're in a thread
 		update_data={}
 		
-		if result>0:
+		if result > 0:
+			update_data['new_entries'] = result
 			if self.is_feed_filter(feed_id):
 				entries = self.get_entrylist(feed_id) #reinitialize filtered_entries dict
 				update_data['unread_count'] = self.get_unread_count(feed_id)
@@ -808,8 +825,14 @@ class ptvDB:
 					
 				update_data['flag_list']=flag_list
 				update_data['pollfail']=False
+			update_data['no_changes'] = False
+		elif result == 0:
+			flag_list = self.get_entry_flags(feed_id)
+			update_data['flag_list']=flag_list
+			update_data['pollfail'] = False
+			update_data['no_changes'] = True
 				
-		return (feed_id,update_data, total)
+		return (feed_id, update_data, total)
 			
 	def poll_feed_trap_errors(self, feed_id, callback):
 		try:
@@ -819,8 +842,7 @@ class ptvDB:
 			feed['feed_id']=feed_id
 			feed['title']=result[0]
 			feed['url']=result[1]
-			self.poll_feed(feed_id)
-			self.reindex()
+			feed['new_entries'] = self.poll_feed(feed_id, A_IGNORE_ETAG+A_DO_REINDEX)
 			callback(feed, True)
 		except Exception, e:#FeedPollError,e:
 			print e
@@ -841,7 +863,7 @@ class ptvDB:
 		"""polls a feed and returns the number of new articles and a flag list.  Optionally, one can pass
 			a feedparser dictionary in the preparsed argument and avoid network operations"""
 		
-		#print "poll:",feed_id, arguments
+		# print "poll:",feed_id, arguments
 		
 		if preparsed is None:
 			#feed_id = self._resolve_pointed_feed(feed_id)
@@ -855,8 +877,13 @@ class ptvDB:
 			data = self._c.fetchone()
 			url,modified,etag=data
 			try:
-				import feedparser
 				feedparser.disableWellFormedCheck=1  #do we still need this?  it used to cause crashes
+				
+				#speed up feedparser
+				if utils.RUNNING_SUGAR:
+					feedparser._sanitizeHTML = lambda a, b: a
+					feedparser._resolveRelativeURIs = lambda a, b, c: a
+				
 				if arguments & A_IGNORE_ETAG == A_IGNORE_ETAG:
 					data = feedparser.parse(url)
 				else:
@@ -904,13 +931,13 @@ class ptvDB:
 
 		if len(data['feed']) == 0 or len(data['items']) == 0:
 			if data.has_key('bozo_exception'):
-				if isinstance(data['bozo_exception'],urllib2.URLError):
+				if isinstance(data['bozo_exception'], URLError):
 					e = data['bozo_exception'][0]
 					errno = e[0]
-					if e[0] in [-3, #failure in name resolution   
+					if errno in [#-2, # Name or service not known 
+								-3, #failure in name resolution   
 								114, #Operation already in progress
 								11]:  #Resource temporarily unavailable
-						print "ioerror",e,"errno:",e[0]
 						raise IOError(e)	
 			
 			if arguments & A_AUTOTUNE == A_AUTOTUNE:
@@ -971,7 +998,7 @@ class ptvDB:
 		if not data.has_key('modified'):
 			modified='0'
 		else:
-			modified = time.mktime(data['modified'])
+			modified = int(time.mktime(data['modified']))
 
 		try:
 			self._db_execute(self._c, u'SELECT title FROM feeds WHERE id=?',(feed_id,))
@@ -1007,36 +1034,20 @@ class ptvDB:
 		self._db.commit()
 		
 		#populate the entries
-		self._db_execute(self._c, """SELECT id,guid,link,title,description FROM entries WHERE feed_id=? order by fakedate""",(feed_id,)) 
+		self._db_execute(self._c, """SELECT id,guid,link,title,description FROM entries WHERE feed_id=? order by fakedate DESC""",(feed_id,)) 
 		existing_entries = self._c.fetchall()
 		
 		#we can't trust the dates inside the items for timing data
 		#bad formats, no dates at all, and timezones screw things up
 		#so I introduce a fake date which works for determining read and
 		#unread article counts, and keeps the articles in order
-		fake_time = time.time()#-len(data['items'])
+		fake_time = int(time.time())
 		i=0
-		try:
-			if data['items'][0].has_key('updated_parsed') == 1:
-				data['items'].sort(lambda x,y: int(time.mktime(y['updated_parsed'])-time.mktime(x['updated_parsed'])))
-		except:
-			try:
-				if data['items'][0].has_key('modified_parsed') == 1:
-					data['items'].sort(lambda x,y: int(time.mktime(y['modified_parsed'])-time.mktime(x['modified_parsed'])))
-			except:
-				try:
-					if data['items'][0].has_key('created_parsed') == 1:
-						data['items'].sort(lambda x,y: int(time.mktime(y['created_parsed'])-time.mktime(x['created_parsed'])))
-				except:	
-					try:	
-						if data['items'][0].has_key('date_parsed') == 1:
-							data['items'].sort(lambda x,y: int(time.mktime(y['date_parsed'])-time.mktime(x['date_parsed'])))
-					except:
-						pass #I feel dirty.
 		
 		new_items = 0
 		
 		flag_list = []
+		not_old = []
 		
 		for item in data['items']:
 			#do a lot of normalizing
@@ -1145,13 +1156,13 @@ class ptvDB:
 			if not item.has_key('date_parsed') or item['date_parsed'] is None:
 				item['date_parsed']=(0,0,0,0,0,0,0,0,0)
 				
-				
-			status = self._get_status(item,existing_entries, feed_id) #FIXME: remove feedid when done debugging!
+			status = self._get_status(item, existing_entries)
 			
 			if status[0]==NEW:
 				new_items = new_items+1
-				self._db_execute(self._c, u'INSERT INTO entries (id, feed_id, title, creator, description, read, fakedate, date, guid, link, old, new) VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "0")',(feed_id,item['title'],item['creator'],item['body'],'0',fake_time-i, time.mktime(item['date_parsed']),item['guid'],item['link'],'0'))
-				self._db_execute(self._c, """SELECT id FROM entries WHERE fakedate=?""",(fake_time-i,))
+				self._db_execute(self._c, u'INSERT INTO entries (id, feed_id, title, creator, description, read, fakedate, date, guid, link, old, new) VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "0")',(feed_id,item['title'],item['creator'],item['body'],'0',fake_time-i, int(time.mktime(item['date_parsed'])),item['guid'],item['link'],'0'))
+
+				self._db_execute(self._c,  "SELECT last_insert_rowid()")
 				entry_id = self._c.fetchone()[0]
 
 				if item.has_key('enclosures'):
@@ -1161,11 +1172,12 @@ class ptvDB:
 						self._db_execute(self._c, u"""INSERT INTO media (id, entry_id, url, mimetype, download_status, viewed, keep, length, feed_id) VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, ?)""", (entry_id, media['url'], media['type'], 0, D_NOT_DOWNLOADED, 0, media['length'], feed_id))
 				self._reindex_entry_list.append(entry_id)
 			elif status[0]==EXISTS:
-				self._db_execute(self._c, """UPDATE entries SET old=0 where id=?""",(status[1],))
+				not_old.append(status[1])
 			elif status[0]==MODIFIED:
 				self._db_execute(self._c, u'UPDATE entries SET title=?, creator=?, description=?, date=?, guid=?, link=?, old=?  WHERE id=?',
 								 (item['title'],item['creator'],item['body'], 
-								 time.mktime(item['date_parsed']),item['guid'],item['link'],'0', status[1]))
+								 int(time.mktime(item['date_parsed'])),item['guid'],
+								 item['link'],'0', status[1]))
 				if self.entry_flag_cache.has_key(status[1]): del self.entry_flag_cache[status[1]]
 				if item.has_key('enclosures'):
 					#self._db_execute(self._c, u'SELECT url FROM media WHERE entry_id=? AND (download_status=? OR download_status=?)',
@@ -1189,33 +1201,32 @@ class ptvDB:
 					#need to add media that's in enclosures but not in db after that process
 					
 					if len(added) > 0:
-						print "found new media in feed", feed_id
-						print "existing set:",db_enc
-						print "enclosures:",item['enclosures']
-						print "I think added:",added
 						for media in item['enclosures']: #add the rest
-							#self._db_execute(self._c, u'SELECT url FROM media WHERE url=?',(media['href'],))
-							#dburl = self._c.fetchone()
-							#if dburl:
 							if media['url'] in added:
 								#if dburl[0] != media['url']: #only add if that url doesn't exist
 								media.setdefault('length', 0)
 								media.setdefault('type', 'application/octet-stream')
 								self._db_execute(self._c, u"""INSERT INTO media (id, entry_id, url, mimetype, download_status, viewed, keep, length, download_date, feed_id) VALUES (NULL, ?, ?, ?, ?, ?, ?, ?, 0, ?)""", (status[1], media['url'], media['type'], 0, D_NOT_DOWNLOADED, 0, media['length'], feed_id))
 								self._db_execute(self._c, u'UPDATE entries SET read=0 WHERE id=?', (status[1],))
-					#self._db.commit()
 				self._reindex_entry_list.append(status[1])
-							
-				#flag_list.append(self.get_entry_flag(status[1]))
 			i+=1
-		self._db.commit()
+
 		#don't call anything old that has media...
 		self._db_execute(self._c, """SELECT entry_id FROM media WHERE download_status>0 AND feed_id=?""",(feed_id,))
-		for id in self._c.fetchall():
-			self._db_execute(self._c, """UPDATE entries SET old=0 WHERE id=?""",(id[0],))
+		result = self._c.fetchall()
+		if result:
+			#combine with EXISTing entries
+			not_old + [r[0] for r in result]
+		if len(not_old) > 0:
+			qmarks = "?,"*(len(not_old)-1)+"?"
+			self._db_execute(self._c, """UPDATE entries SET old=0 WHERE id in (""" +
+							 qmarks + ')', tuple(not_old))
+		
 		self._db.commit()
-		#anything not set above as new, mod, or exists is no longer in
-		#the xml and therefore should be deleted
+		
+		# anything not set above as new, mod, or exists is no longer in
+		# the xml and therefore could be deleted if we have more articles than 
+		# the limit
 		
 		self._db_execute(self._c, """SELECT count(*) FROM entries WHERE feed_id=?""",(feed_id,))
 		all_entries = self._c.fetchone()[0]
@@ -1268,7 +1279,7 @@ class ptvDB:
 		
 		self._db_execute(self._c, u'SELECT lastpoll, newatlast, pollfreq FROM feeds WHERE id=?',(feed_id,))
 		last_time,newatlast,old_poll_freq = self._c.fetchone()
-		cur_time = time.time()
+		cur_time = int(time.time())
 		#this could suck if the program was just started, so only do it if the poll_freq seems correct
 		#however still update the db with the poll time
 		self._db_execute(self._c, u'UPDATE feeds SET lastpoll=?, newatlast=? WHERE id=?',(cur_time,new_items,feed_id))
@@ -1284,7 +1295,7 @@ class ptvDB:
 			#this algorithm seems to be the most accurate based on my own personal judgment
 			self._db_execute(self._c, 'SELECT date FROM entries WHERE feed_id=?',(feed_id,))
 			datelist = self._c.fetchall()
-			datelist.append((time.time(),)) #helps in some cases to pretend we found one now
+			datelist.append((int(time.time()),)) #helps in some cases to pretend we found one now
  			i=0
 			list=[]
 			for item in datelist[:-1]:
@@ -1311,7 +1322,7 @@ class ptvDB:
 		self._db_execute(self._c, 'UPDATE feeds SET pollfreq=? WHERE id=?',(poll_freq,feed_id))
 		self._db.commit()
 		
-	def _get_status(self, item, existing_entries, feed_id):
+	def _get_status(self, item, existing_entries):
 		"""returns status, the entry_id of the matching entry (if any), and the media list if unmodified"""
 		ID=0
 		GUID=1
@@ -1320,32 +1331,36 @@ class ptvDB:
 		BODY=4
 
 		entry_id=-1
-		old_hash = sha.new()
-		new_hash = sha.new()
+		
+		t_item = {'guid': item['guid'],
+				'body': item['body'],
+				'link': item['link'],
+				'title': item['title']}
+				
 		
 		for entry_item in existing_entries:
-			if str(item['guid'])!='0':
-				if str(entry_item[GUID]) == str(item['guid']):# and entry_item[TITLE] == item['title']:
+			if len(str(t_item['guid'])) > 2: #even 3 chars for a guid seems small, but oh well
+				if str(entry_item[GUID]) == str(t_item['guid']):# and entry_item[TITLE] == t_item['title']:
 					entry_id = entry_item[ID]
-					old_hash.update(self._ascii(entry_item[GUID])+self._ascii(entry_item[BODY]))
-					new_hash.update(self._ascii(item['guid'])+self._ascii(item['body']))
+					old_hash = self._ascii(entry_item[GUID])+self._ascii(entry_item[BODY])
+					new_hash = self._ascii(t_item['guid'])+self._ascii(t_item['body'])
 					break
-			elif item['link']!='':
-				if entry_item[LINK] == item['link'] and entry_item[TITLE] == item['title']:
+			elif t_item['link']!='':
+				if entry_item[LINK] == t_item['link'] and entry_item[TITLE] == t_item['title']:
 					entry_id = entry_item[ID]
-					old_hash.update(self._ascii(entry_item[LINK])+self._ascii(entry_item[BODY]))
-					new_hash.update(self._ascii(item['link'])+self._ascii(item['body']))
+					old_hash = self._ascii(entry_item[LINK])+self._ascii(entry_item[BODY])
+					new_hash = self._ascii(t_item['link'])+self._ascii(t_item['body'])
 					break
-			elif entry_item[TITLE] == item['title']:
+			elif entry_item[TITLE] == t_item['title']:
 				entry_id = entry_item[ID]
-				old_hash.update(self._ascii(entry_item[TITLE])+self._ascii(entry_item[BODY]))
-				new_hash.update(self._ascii(item['title'])+self._ascii(item['body']))
+				old_hash = self._ascii(entry_item[TITLE])+self._ascii(entry_item[BODY])
+				new_hash = self._ascii(t_item['title'])+self._ascii(t_item['body'])
 				break
 
 		if entry_id == -1:
 			return (NEW, -1, [])
 
-		if new_hash.hexdigest() == old_hash.hexdigest():
+		if new_hash == old_hash:
 			#now check enclosures
 			old_media = self.get_entry_media(entry_id)
 			if old_media is None:
@@ -1407,6 +1422,37 @@ class ptvDB:
 			media_list.append(medium)			
 		return media_list
 		
+	def get_entry_media_block(self, entry_list):
+		if len(entry_list) == 0:
+			return
+		qmarks = "?,"*(len(entry_list)-1)+"?"
+		
+		self._db_execute(self._c, """SELECT id,entry_id,url,file,download_status,viewed,length,mimetype FROM media WHERE entry_id in ("""+qmarks+')',tuple(entry_list))
+		result = self._c.fetchall()
+		if result is None:
+			return []
+		media_dict = {}
+		for datum in result:
+			medium={}
+			medium['url']=datum[2] #MAGIC
+			medium['download_status']=int(datum[4]) #MAGIC
+			try:
+				medium['size']=int(datum[6]) #MAGIC
+			except:
+				medium['size']=0
+			medium['media_id']=int(datum[0]) #MAGIC
+			medium['file']=datum[3] #MAGIC			
+			medium['entry_id']=datum[1] #MAGIC
+			medium['viewed']=int(datum[5]) #MAGIC
+			medium['mimetype']=datum[7] #MAGIC
+			
+			if not media_dict.has_key(medium['entry_id']):
+				media_dict[medium['entry_id']] = [medium]
+			else:
+				media_dict[medium['entry_id']].append(medium)				
+		
+		return media_dict
+		
 	def get_media(self, media_id):
 		self._db_execute(self._c, u'SELECT url, download_status, length, file, entry_id, viewed, mimetype FROM media WHERE id=?',(media_id,))
 		datum=self._c.fetchone()
@@ -1431,7 +1477,7 @@ class ptvDB:
 		return self._c.fetchone()[0]
 	
 	def get_entry(self, entry_id):
-		self._db_execute(self._c, """SELECT title, creator, link, description, feed_id, date, read FROM entries WHERE id=?""",(entry_id,))
+		self._db_execute(self._c, """SELECT title, creator, link, description, feed_id, date, read FROM entries WHERE id=? LIMIT 1""",(entry_id,))
 		result = self._c.fetchone()
 		
 		entry_dic={}
@@ -1448,6 +1494,28 @@ class ptvDB:
 			raise NoEntry, entry_id
 		return entry_dic
 		
+	def get_entry_block(self, entry_list):
+		if len(entry_list) == 0:
+			return
+		qmarks = "?,"*(len(entry_list)-1)+"?"
+		self._db_execute(self._c, u'SELECT title, creator, link, description, feed_id, date, read, id FROM entries WHERE id in ('+qmarks+')', (tuple(entry_list)))
+		result = self._c.fetchall()
+		if result is None:
+			return []
+		retval = []
+		for entry in result:
+			entry_dic = {}
+			entry_dic['title'] = entry[0]
+			entry_dic['creator'] = entry[1]
+			entry_dic['link'] = entry[2]
+			entry_dic['description']=entry[3]
+			entry_dic['feed_id']= entry[4]
+			entry_dic['date'] = entry[5]
+			entry_dic['read'] = entry[6]
+			entry_dic['entry_id'] = entry[7]
+			retval.append(entry_dic)
+		return retval
+		
 	def get_entrylist(self, feed_index):
 		self._db_execute(self._c, u'SELECT feed_pointer,description FROM feeds WHERE id=?',(feed_index,))
 		result = self._c.fetchone()
@@ -1464,7 +1532,12 @@ class ptvDB:
 			#gonna be slow :(
 			for entry_id,title, fakedate, feed_id in s_entries:
 				self._db_execute(self._c, """SELECT read FROM entries WHERE id=? LIMIT 1""",(entry_id,))
-				readinfo = self._c.fetchone()[0]
+				try:
+					readinfo = self._c.fetchone()[0]
+				except:
+					print "error in search results, reindexing"
+					readinfo = 0
+					self.reindex()
 				entries.append([entry_id, title, fakedate, readinfo, feed_id])
 			self._filtered_entries[feed_index] = entries
 			return entries
@@ -1506,7 +1579,6 @@ class ptvDB:
 		except: return None
 		
 	def get_feed_info(self, feed_id):
-		import urlparse
 		self._db_execute(self._c, """SELECT title, description, url, link, feed_pointer, lastpoll, pollfreq FROM feeds WHERE id=?""",(feed_id,))
 		try:
 			result = self._c.fetchone()
@@ -1542,7 +1614,6 @@ class ptvDB:
 			url=self._c.fetchone()[0]
 			
 			try:
-				import feedparser
 				feedparser.disableWellFormedCheck=1
 				data = feedparser.parse(url)
 			except:
@@ -1573,7 +1644,7 @@ class ptvDB:
 				
 	def set_media_download_status(self, media_id, status):
 		if status == D_DOWNLOADED:
-			self._db_execute(self._c, u'UPDATE media SET download_status=?, download_date=? WHERE id=?', (status,time.time(),media_id,))
+			self._db_execute(self._c, u'UPDATE media SET download_status=?, download_date=? WHERE id=?', (status, int(time.time()),media_id,))
 			self._db.commit()
 		else:
 			self._db_execute(self._c, u'UPDATE media SET download_status=? WHERE id=?', (status,media_id,))
@@ -1634,7 +1705,7 @@ class ptvDB:
 			if self.entry_flag_cache.has_key(e): del self.entry_flag_cache[e]
 		
 	def get_entry_read(self, entry_id):
-		self._db_execute(self._c, u'SELECT read FROM entries WHERE id=?',(entry_id,))
+		self._db_execute(self._c, u'SELECT read FROM entries WHERE id=? LIMIT 1',(entry_id,))
 		retval = self._c.fetchone()[0]
 		return int(retval)
 		
@@ -1818,27 +1889,15 @@ class ptvDB:
 		   it up if I do my own sort.  profilers don't lie!"""
 		feed_info = {}
 		
-		is_filter = self.is_feed_filter(feed_id)
+		is_filter = False
+		if utils.HAS_LUCENE:
+			is_filter = self.is_feed_filter(feed_id)
 		
 		if is_filter or self.cache_dirty:
-			if is_filter:
-				entries = self.get_entrylist(feed_id) #gets search...
-				read_status = []
-				for e in entries:
-					self._db_execute(self._c, u'SELECT read FROM entries WHERE id=?',(e[0],))
-					result = self._c.fetchone()
-					if result: read_status.append(result)
-				entry_list = read_status
-			else:
-				self._db_execute(self._c, """SELECT read FROM entries WHERE feed_id=?""",(feed_id,))
-				entry_list = self._c.fetchall()
-			unread=0
-			for item in entry_list:
-				if item[0]==0: #read
-					unread=unread+1
-			feed_info['unread_count'] = unread
-			feed_info['entry_count'] = len(entry_list)
-			feed_info['important_flag'] = self.get_feed_flag(feed_id)  #not much speeding up this	
+			flaglist = self.get_entry_flags(feed_id)
+			feed_info['important_flag'] = self.get_feed_flag(feed_id, flaglist)  #not much speeding up this	
+			feed_info['entry_count'] = len(flaglist)
+			feed_info['unread_count'] = len([f for f in flaglist if f & F_UNVIEWED])
 		else:
 			self._db_execute(self._c, u'SELECT flag_cache, unread_count_cache, entry_count_cache FROM feeds WHERE id=?',(feed_id,))
 			cached_info = self._c.fetchone()
@@ -1883,26 +1942,27 @@ class ptvDB:
 					status = D_DOWNLOADED
 					break
 		
-		if status==D_ERROR:
-			importance=importance+F_ERROR
-		if status==D_DOWNLOADING:
-			importance=importance+F_DOWNLOADING		
+		if status == D_ERROR:
+			importance = importance + F_ERROR
+		if status == D_DOWNLOADING:
+			importance = importance + F_DOWNLOADING		
 			
 		if medialist:	
-			importance=importance+F_MEDIA
-			if status==D_DOWNLOADED:
-				importance=importance+F_DOWNLOADED
-			elif status==D_RESUMABLE:
-				importance=importance+F_PAUSED
+			importance = importance + F_MEDIA
+			if status == D_DOWNLOADED:
+				importance = importance + F_DOWNLOADED
+			elif status == D_RESUMABLE:
+				importance = importance + F_PAUSED
 			for medium in medialist:
-				if medium['viewed']==0:
-					importance=importance+F_UNVIEWED
+				if medium['viewed'] == 0:
+					importance = importance + F_UNVIEWED
 					break
 		else:
-			if int(read)==0:
-				importance=importance+F_UNVIEWED
+			if int(read) == 0:
+				importance = importance + F_UNVIEWED
 		
-		self.entry_flag_cache[entry_id] = importance
+		if USING_FLAG_CACHE:
+			self.entry_flag_cache[entry_id] = importance
 		return importance		
 		
 	def get_unread_count(self, feed_id):
@@ -1956,15 +2016,16 @@ class ptvDB:
 			if self.get_feed_media_count(feed_id) == 0:
 				medialist = []
 			for entry,read in entrylist:
-				flaglist.append(self.get_entry_flag(entry,read=read, medialist=medialist))
+				flaglist.append(self.get_entry_flag(entry, read=read, medialist=medialist))
 		return flaglist
 	
-	def get_feed_flag(self, feed_id):
+	def get_feed_flag(self, feed_id, flaglist = None):
 		""" Based on a feed, what flag best represents the overall status of the feed at top-level?
 			This is based on the numeric value of the flag, which is why flags are enumed the way they are."""
 			
 		feed_has_media=0
-		flaglist = self.get_entry_flags(feed_id)
+		if flaglist is None:
+			flaglist = self.get_entry_flags(feed_id)
 		
 		if len(flaglist)==0:
 			return 0
@@ -2111,7 +2172,10 @@ class ptvDB:
 			item = OPML.Outline()
 			item['title']=self._ascii(feed[0])
 			item['text']=self._ascii(feed[0])
-			item['description']=self._ascii(feed[1])
+			if feed[1] is None: 
+				item['description'] = ""
+			else:
+				item['description'] = self._ascii(feed[1])
 			item['xmlUrl']=feed[2]
 			o.outlines.append(item)
 		o.output(stream)
@@ -2162,14 +2226,23 @@ class ptvDB:
 		else: #just a list in a file
 			url_list = []
 			count = 0
-			for url in stream.readlines():
+			for line in stream.readlines():
+				line = line.strip()
+				if len(line) == 0: continue
+				space_at = line.find(' ')
+				if space_at >= 0:
+					url = line[:space_at]
+					title = line[space_at+1:]
+				else:
+					url = line
+					title = None
 				count+=1
-				url_list.append(url)
+				url_list.append((url, title))
 			stream.close()
 			yield (1,len(url_list))
-			for url in url_list:
+			for url, title in url_list:
 				try:
-					feed_id=self.insertURL(url)
+					feed_id=self.insertURL(url, title)
 					yield (1,feed_id)
 				except FeedAlreadyExists, f:
 					yield (0,f.feed)
