@@ -1,6 +1,7 @@
 # Written by Owen Williams
 # see LICENSE for license information
-import os,sys,time, pwd
+import os,sys,time, pwd, os.path
+import subprocess
 import string
 import fnmatch
 import urllib
@@ -8,6 +9,9 @@ import HTMLParser
 import string
 import locale
 import gettext
+import shutil
+
+import gtk
 
 from subprocess import Popen, PIPE, STDOUT
 
@@ -26,7 +30,24 @@ if RUNNING_SUGAR:
 	HAS_GCONF = False
 	HAS_GNOMEVFS = False
 	HAS_PYXML = False
+	HAS_STATUS_ICON = False
+	HAS_MOZILLA = True
+	#HAS_GSTREAMER = True
+	#I'm having trouble building gst for jhbuild, so detect this
+	try:
+		import pygst
+		pygst.require("0.10")
+		import gst
+		HAS_GSTREAMER = True
+	except:
+		HAS_GSTREAMER = False
 else:
+	try:
+		import gtkmozembed
+		HAS_MOZILLA = True
+	except:
+		HAS_MOZILLA = False
+
 	try:
 		import PyLucene
 		HAS_LUCENE = True
@@ -51,18 +72,36 @@ else:
 	except:
 		HAS_PYXML = False
 		
-try:
-	import pygst
-	pygst.require("0.10")
-	import gst
-	HAS_GSTREAMER = True
-except:
-	HAS_GSTREAMER = False
+	if gtk.pygtk_version >= (2, 10, 0):
+		HAS_STATUS_ICON = True
+	else:
+		HAS_STATUS_ICON = False
+		
+	try:
+		import pynotify
+		HAS_PYNOTIFY = True
+	except:
+		HAS_PYNOTIFY = False
+
+	try:
+		import pygst
+		pygst.require("0.10")
+		import gst
+		HAS_GSTREAMER = True
+	except:
+		HAS_GSTREAMER = False
 	
-VERSION="2.80"
+VERSION="3.0"
 #DEBUG
-_USE_KDE_OVERRIDE=False
-#HAS_LUCENE = False
+#_USE_KDE_OVERRIDE=False
+# Lucene sucks, forget it
+HAS_LUCENE = False
+# Pynotify is still broken, forget it
+HAS_PYNOTIFY = False
+#HAS_PYXML = False
+#HAS_STATUS_ICON = False
+#HAS_GNOMEVFS = False
+#HAS_MOZILLA=False
 
 def format_size(size):
 	if size > 1073741824:
@@ -77,6 +116,37 @@ def format_size(size):
 def GetPrefix():
 	h, t = os.path.split(os.path.split(os.path.abspath(sys.argv[0]))[0])
 	return h
+	
+def get_glade_prefix():
+	import utils
+	for p in (os.path.join(GetPrefix(),"share","penguintv"),
+		  os.path.join(GetPrefix(),"share"),
+		  os.path.join(os.path.split(os.path.abspath(sys.argv[0]))[0],"share"),
+		  os.path.join(GetPrefix(),"share","sugar","activities","ptv","share"),
+		  os.path.join(os.path.split(os.path.split(utils.__file__)[0])[0],'share')):
+		try:
+			os.stat(os.path.join(p,"penguintv.glade"))
+			return p
+		except:
+			continue
+	return None
+	
+def get_icon_filename():
+	try:
+		icon_file = GetPrefix()+"/share/pixmaps/penguintvicon.png"
+		os.stat(icon_file)
+	except:
+		try:
+			icon_file = GetPrefix()+"/share/penguintvicon.png" #in case the install is still in the source dirs
+			os.stat(icon_file)
+		except:
+			try:
+				icon_file = get_glade_prefix()+"/penguintvicon.png"
+				os.stat(icon_file)
+			except Exception, e:
+				print "icon not found"
+				raise e
+	return icon_file		
 
 def hours(n):  #this func copyright Bram Cohen
     if n == -1:
@@ -384,45 +454,21 @@ class StrippingParser(HTMLParser.HTMLParser):
 #    parser.cleanup()
 #    return parser.result
 
-class AltParser(HTMLParser.HTMLParser):
-	def __init__(self):
-		HTMLParser.HTMLParser.__init__(self)
-		self.alt_tags={}
-		self.head_end=False
-		
-	def handle_starttag(self, tag, attrs):
-		"""Signal when we get to a tag."""
-		if tag=='link':
-			attr_dic = {}
-			for attr in attrs:
-				attr_dic[attr[0]] = attr[1]
-			try:
-				if attr_dic['rel'] == 'alternate':
-					if attr_dic['type'] in ['application/atom+xml','application/rss+xml','text/xml']:
-						self.alt_tags[attr_dic['type']] = attr_dic['href']
-			except:
-				pass
-
-	def handle_endtag(self, tag):
-		if tag == 'head':
-			self.head_end=True
-
-		
-#http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/457667
-#I know this is very bad, but damn if it doesn't work
-import __main__
-
-class SuperGlobal:
-
-    def __getattr__(self, name):
-        return __main__.__dict__.get(name, None)
-        
-    def __setattr__(self, name, value):
-        __main__.__dict__[name] = value
-        
-    def __delattr__(self, name):
-        if __main__.__dict__.has_key(name):
-            del  __main__.__dict__[name]
+##http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/457667
+##I know this is very bad, but damn if it doesn't work
+#import __main__
+#
+#class SuperGlobal:
+#
+#    def __getattr__(self, name):
+#        return __main__.__dict__.get(name, None)
+#        
+#    def __setattr__(self, name, value):
+#        __main__.__dict__[name] = value
+#        
+#    def __delattr__(self, name):
+#        if __main__.__dict__.has_key(name):
+#            del  __main__.__dict__[name]
             
 #thanks http://www.peterbe.com/plog/html-entity-fixer
 #from htmlentitydefs import entitydefs
@@ -450,45 +496,220 @@ def html_entity_unfixer(text):
 
 def get_disk_free(f="/"):
 	
-	"""returns free disk space in kilobytes for 
+	"""returns free disk space in bytes for 
 	   the disk that contains file f, defaulting to root.  
 	   Returns 0 on error"""
 	
-	os.stat(f) #if this fails it will raise exactly the right error
+	stats = os.statvfs(f) #if this fails it will raise exactly the right error
 	
-	cmd = "df -k " + f
+	return stats.f_bsize * stats.f_bavail
 	
-	p = Popen(cmd, shell=True, 
-		      stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
-						 
-	lines = p.stdout.readlines()
-	
-	try:
-		return int(lines[1].split()[3]) * 1024
-	except:
-		return 0
 		
 def get_disk_total(f="/"):
 	
-	"""returns total disk space in kilobytes for 
+	"""returns total disk space in bytes for 
 	   the disk that contains file f, defaulting to root.
 	   Returns 0 on error"""
 	
-	os.stat(f) #if this fails it will raise exactly the right error
+	stats = os.statvfs(f) #if this fails it will raise exactly the right error
 	
-	cmd = "df -k " + f
+	return stats.f_bsize * stats.f_blocks
+		
+def init_gtkmozembed():
+	"""We need to set up mozilla with set_comp_path in order for it not to 
+	crash.  The fun part is not hardcoding that path since we have no way
+	of getting it from the module itself.  good luck with this"""
+
+	assert HAS_MOZILLA
+	os.chdir(os.getenv('HOME'))
+	cmd = "ldd " + gtkmozembed.__file__ + "  | grep xpcom.so"
+	p = subprocess.Popen(cmd, shell=True, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+	retval = p.wait()
+	stderr = p.stderr.read()
+	if len(stderr) > 1 or retval != 0:
+		return False
+	ldd_output = p.stdout.read()
+	comp_path = os.path.split(ldd_output.split()[2])[0]
 	
-	p = Popen(cmd, shell=True, 
-		      stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
-						 
-	lines = p.stdout.readlines()
+	_init_mozilla_proxy()
+	
+	print "initializing mozilla in", comp_path
+	gtkmozembed.set_comp_path(comp_path)
+	
+	return True
+	
+def _init_mozilla_proxy():
+	if RUNNING_SUGAR:
+		# don't even try
+		return
+		
+	home = os.path.join(os.getenv('HOME'), ".penguintv")
+	_mkdir(os.path.join(home, "gecko"))
+	
+	sys_proxy = {}
+	sys_proxy['host'] = ""
+	sys_proxy['port'] = 0
+	sys_proxy['type'] = 0
+	sys_proxy['autoconfig_url'] = ""
+	# get system proxy prefs if any
+	#if True:
+	#	sys_proxy['host'] = "6.2.7.2"
+	#	sys_proxy['port'] = 8080
+	#	sys_proxy['type'] = 1
+	#	sys_proxy['autoconfig_url'] = "testing"
+	if HAS_GCONF:
+		# get gnome http proxy preferences
+		conf = gconf.client_get_default()
+		use_proxy = conf.get_bool("/system/http_proxy/use_http_proxy")
+		if use_proxy:
+			sys_proxy['host'] = conf.get_string("/system/http_proxy/host")
+			sys_proxy['port'] = conf.get_int("/system/http_proxy/port")
+			sys_proxy['type'] = 1
+	else:
+		# get most-recently modified moz prefs. 
+		prefs_files = []
+		for f in GlobDirectoryWalker(os.path.join(os.getenv('HOME'), ".mozilla"), "prefs.js"):
+			prefs_files.append((os.stat(f).st_mtime, f))
+		if len(prefs_files) != 0:
+			prefs_files.sort()
+			prefs_files.reverse()
+			source_prefs = prefs_files[0][1]
+			sys_proxy = _get_proxy_prefs(source_prefs)
+		
+	# check against current settings
+	try:
+		os.stat(os.path.join(home, "gecko", "prefs.js"))
+		cur_proxy = _get_proxy_prefs(os.path.join(home, "gecko", "prefs.js"))
+		if sys_proxy == cur_proxy:
+			print "gecko proxy settings up to date"
+			return
+	except:
+		pass
+
+	try:
+		print "updating gecko proxy settings"
+		f = open(os.path.join(home, "gecko", "prefs.js"), "w")
+		f.write("""# Mozilla User Preferences
+
+	/* Do not edit this file.
+	 *
+	 * If you make changes to this file while the application is running,
+	 * the changes will be overwritten when the application exits.
+	 *
+	 * To make a manual change to preferences, you can visit the URL about:config
+	 * For more information, see http://www.mozilla.org/unix/customizing.html#prefs
+	 */
+
+	user_pref("network.proxy.type", %d);
+	user_pref("network.proxy.http", "%s");
+	user_pref("network.proxy.http_port", %d);
+	user_pref("network.proxy.autoconfig_url", "%s");
+	""" % (sys_proxy['type'], sys_proxy['host'], sys_proxy['port'], sys_proxy['autoconfig_url']))
+		f.close()
+	except:
+		print "WARNING: couldn't create prefs.js, proxy server connections may not work"		
+		
+def _get_proxy_prefs(filename):
+	def isNumber(x):
+		try:
+			float(x)
+			return True
+		except:
+			return False
+	
+	proxy = {}
+	proxy['host'] = ""
+	proxy['port'] = 0
+	proxy['type'] = 0
+	proxy['autoconfig_url'] = ""
 	
 	try:
-		return int(lines[1].split()[1]) * 1024
+		f = open(filename, "r")
 	except:
-		return 0
+		print "WARNING: couldn't open gecko preferences file ", filename
+		return proxy
+	
+	for line in f.readlines():
+		if 'network' in line:
+			if '"network.proxy.http"' in line:
+				proxy['host'] = line.split('"')[3]
+			elif '"network.proxy.autoconfig_url"' in line:
+				proxy['autoconfig_url'] = line.split('"')[3]
+			elif '"network.proxy.http_port"' in line:
+				proxy['port'] = int("".join([c for c in line.split('"')[2] if isNumber(c)]))
+			elif '"network.proxy.type"' in line:
+				proxy['type'] = int("".join([c for c in line.split('"')[2] if isNumber(c)]))
+				
+	f.close()
+	return proxy
+	
+def get_pynotify_ok():
+	if not HAS_PYNOTIFY:
+		#print "pynotify not found, using fallback notifications"
+		return False
 
+	# first get what package config reports
+	os.chdir(os.getenv('HOME'))
+	
+	cmd = "pkg-config notify-python --modversion"
+	p = subprocess.Popen(cmd, shell=True, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+	retval = p.wait()
+	stderr = p.stderr.read()
+	if len(stderr) > 1 or retval != 0:
+		print "trouble getting notify-python version from pkg-config, using fallback notifications"
+		return False
+	major,minor,rev = p.stdout.read().split('.')
+	
+	major = int(major)
+	minor = int(minor)
+	rev = int(rev)
 
+	# if it's bad, return false
+	if minor < 1:
+		print "pynotify too old, using fallback notifications"
+		return False
+	if minor == 1 and rev == 0:
+		print "pynotify too old, using fallback notifications"
+		return False
+
+	# if it's good, check to see it's not lying about prefix
+	cmd = "pkg-config notify-python --variable=prefix"
+	p = subprocess.Popen(cmd, shell=True, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+	retval = p.wait()
+	stderr = p.stderr.read()
+	if len(stderr) > 1 or retval != 0:
+		print "trouble getting notify-python prefix from pkg-config, using fallback notifications"
+		return False
+	pkgconfig_prefix = p.stdout.read().strip()
+	
+	try:
+		dirname = os.path.split(pynotify.__file__)[0]
+		f = open(os.path.join(dirname, "_pynotify.la"))
+	except:
+		print "trouble opening _pynotify.la, using fallback notifications"
+		return False
+	
+	libdir_line = ""
+	
+	for line in f.readlines():
+		if line[0:6] == "libdir":
+			libdir_line = line.split("=")[1][1:-1]
+			break
+	f.close()
+	
+	libdir_line = libdir_line.strip()
+	
+	if len(libdir_line) == 0:
+		print "trouble reading _pynotify.la, using fallback notifications"
+		return False
+	
+	if pkgconfig_prefix not in libdir_line:
+		print "pkgconfig does not agree with _pynotify.la, using fallback notifications"
+		return False
+
+	print "using pynotify notifications"
+	return True
+	
 if is_kde():
 	import kio
 
